@@ -1,20 +1,20 @@
 /*!
- * \file        chan_sccp.h
+ * \file	chan_sccp.h
  * \brief       Chan SCCP Main Class
  *
  * An implementation of Skinny Client Control Protocol (SCCP)
  *
- * \author      Sergio Chersovani <mlists [at] c-net.it>
+ * \author	Sergio Chersovani <mlists [at] c-net.it>
  * \brief       Main chan_sccp Class
- * \note        Reworked, but based on chan_sccp code.
- *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *              Modified by Jan Czmok and Julien Goodwin
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \note	Reworked, but based on chan_sccp code.
+ *		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *		Modified by Jan Czmok and Julien Goodwin
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *		See the LICENSE file at the top of the source tree.
  * \warning     File has been Lined up using 8 Space TABS
  *
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $
+ * $Date: 2014-05-02 09:48:10 +0200 (Fri, 02 May 2014) $
+ * $Revision: 5398 $
  */
 
 #ifndef __CHAN_SCCP_H
@@ -34,11 +34,8 @@ extern "C" {
 #include <config.h>
 //#include "common.h"
 
-#define ENUMMACRO_FILE "chan_sccp_enums.hh"
-#define ENUMMACRO_GENERATE ENUMMACRO_GENHEADER
 #include "sccp_enum_macro.h"
-#undef  ENUMMACRO_GENERATE
-#undef  ENUMMACRO_FILE
+#include "chan_sccp_enums.hh"
 
 #define sccp_mutex_t ast_mutex_t
 
@@ -115,15 +112,12 @@ char SCCP_REVISIONSTR[30];
 #define SCCP_FIRST_SPEEDDIALINSTANCE 1										/* whats the instance of the first speeddial button */
 
 #define SCCP_SOCKET_ACCEPT_TIMEOUT 2000
-#define SCCP_DISPLAYSTATUS_TIMEOUT 5
 
 /* Simulated Enbloc Dialing */
 #define SCCP_SIM_ENBLOC_DEVIATION 3.5
 #define SCCP_SIM_ENBLOC_MAX_PER_DIGIT 400
 #define SCCP_SIM_ENBLOC_MIN_DIGIT 3
-#define SCCP_SIM_ENBLOC_TIMEOUT 2
-
-#define SCCP_HANGUP_TIMEOUT 15000
+#define SCCP_SIM_ENBLOC_TIMEOUT 2000
 
 #define THREADPOOL_MIN_SIZE 2
 #define THREADPOOL_MAX_SIZE 10
@@ -132,16 +126,12 @@ char SCCP_REVISIONSTR[30];
 #define CHANNEL_DESIGNATOR_SIZE 32
 #define SCCP_TIME_TO_KEEP_REFCOUNTEDOBJECT 2000									// ms
 
-#define DEFAULT_PBX_STR_BUFFERSIZE 512
-
 /*! \todo I don't like the -1 returned value */
 #define sccp_true(x) (pbx_true(x) ? 1 : 0)
 #define sccp_false(x) (pbx_false(x) ? 1 : 0)
 
 // When DEBUGCAT_HIGH is set, we use ast_log instead of ast_verbose
 #define sccp_log1(...) { if ((sccp_globals->debug & (DEBUGCAT_FILELINEFUNC)) == DEBUGCAT_FILELINEFUNC) { ast_log(AST_LOG_NOTICE, __VA_ARGS__); } else { ast_verbose(__VA_ARGS__); } }
-//#define sccp_log1(...) { ast_log(AST_LOG_DEBUG, __VA_ARGS__); ast_verbose(__VA_ARGS__); }
-//#define sccp_log1(...) { ast_verbose(__VA_ARGS__); }
 #define sccp_log(_x) if ((sccp_globals->debug & (_x))) sccp_log1
 #define sccp_log_and(_x) if ((sccp_globals->debug & (_x)) == (_x)) sccp_log1
 
@@ -243,7 +233,7 @@ typedef struct sccp_line sccp_line_t;										/*!< SCCP Line Structure */
 typedef struct sccp_speed sccp_speed_t;										/*!< SCCP Speed Structure */
 typedef struct sccp_service sccp_service_t;									/*!< SCCP Service Structure */
 typedef struct sccp_device sccp_device_t;									/*!< SCCP Device Structure */
-typedef struct sccp_addon sccp_addon_t;										/*!< SCCP Add-On Structure */
+typedef struct sccp_addon sccp_addon_t;										/*!< SCCP Add-On Structure */// Added on SVN 327 -FS
 typedef struct sccp_hint sccp_hint_t;										/*!< SCCP Hint Structure */
 typedef struct sccp_hostname sccp_hostname_t;									/*!< SCCP HostName Structure */
 
@@ -256,7 +246,6 @@ typedef struct sccp_buttonconfig sccp_buttonconfig_t;								/*!< SCCP Button Co
 typedef struct sccp_hotline sccp_hotline_t;									/*!< SCCP Hotline Structure */
 typedef struct sccp_callinfo sccp_callinfo_t;									/*!< SCCP Call Information Structure */
 typedef struct sccp_linedevices sccp_linedevices_t;								/*!< SCCP Line Connected to Devices */
-typedef struct softKeySetConfiguration sccp_softKeySetConfiguration_t;						/*!< SoftKeySet configuration */
 
 #ifndef SOLARIS
 typedef enum { FALSE = 0, TRUE = 1 } boolean_t;									/*!< Asterisk Reverses True and False; nice !! */
@@ -266,6 +255,7 @@ typedef enum { FALSE = 0, TRUE = 1 } boolean_t;									/*!< Asterisk Reverses T
 #define TRUE B_TRUE
 #endif
 typedef void sk_func(sccp_device_t * d, sccp_line_t * l, sccp_channel_t * c);
+typedef enum { ANSWER_LAST_FIRST = 1, ANSWER_OLDEST_FIRST = 2 } call_answer_order_t;				/*!< Enum Call Answer Order */
 typedef enum { PARK_RESULT_FAIL = 0, PARK_RESULT_SUCCESS = 1 } sccp_parkresult_t;				/*!< Enum Park Result */
 typedef enum {
 	/* *INDENT-OFF* */
@@ -510,7 +500,7 @@ static const struct sccp_feature_type {
 	{SCCP_FEATURE_DO_NOT_DISTURB,		"do not disturb"},
 	{SCCP_FEATURE_CONF_LIST,		"ConfList"},
 	{SCCP_FEATURE_REMOVE_LAST_PARTICIPANT,	"RemoveLastParticipant"},
-	{SCCP_FEATURE_HLOG, 			"Hunt Group Log-in/out"},
+	{SCCP_FEATURE_HLOG, 			"Hlog"},
 	{SCCP_FEATURE_QRT, 			"QRT"},
 	{SCCP_FEATURE_CALLBACK, 		"CallBack"},
 	{SCCP_FEATURE_OTHER_PICKUP, 		"OtherPickup"},
@@ -656,6 +646,7 @@ typedef enum {
 } sccp_call_statistics_type_t;
 
 typedef struct sccp_call_statistic {
+	char type[8];
 	uint32_t num;
 	uint32_t packets_sent;
 	uint32_t packets_received;
@@ -708,6 +699,7 @@ struct composedId {
 	struct subscriptionId subscriptionId;
 };
 
+
 /*!
  * \brief SCCP Line-Devices Structure
  */
@@ -806,9 +798,9 @@ struct sccp_devstate_specifier {
  * \note A line is the equivalent of a 'phone line' going to the phone.
  */
 struct sccp_line {
-	//sccp_mutex_t lock;                                                                                      /*!< Asterisk: Lock Me Up and Tie me Down */
+//	sccp_mutex_t lock;											/*!< Asterisk: Lock Me Up and Tie me Down */
 	char id[8];												/*!< This line's ID, used for logging into (for mobility) */
-	char name[StationMaxNameSize];										/*!< The name of the line, so use in asterisk (i.e SCCP/[name]) */
+	char name[StationMaxNameSize];												/*!< The name of the line, so use in asterisk (i.e SCCP/[name]) */
 #ifdef CS_SCCP_REALTIME
 	boolean_t realtime;											/*!< is it a realtimeconfiguration */
 #endif
@@ -835,14 +827,15 @@ struct sccp_line {
 	char *namedpickupgroup;											/*!< Named Pickup Group */
 #endif
 
-	char cid_num[SCCP_MAX_EXTENSION];									/* smaller would be better (i.e. 32) *//*!< Caller(ID) to use on outgoing calls  */
-	char cid_name[SCCP_MAX_EXTENSION];									/* smaller would be better (i.e. 32) *//*!< Caller(Name) to use on outgoing calls */
+	char cid_num[SCCP_MAX_EXTENSION];	/* smaller would be better (i.e. 32) */				/*!< Caller(ID) to use on outgoing calls  */
+	char cid_name[SCCP_MAX_EXTENSION];	/* smaller would be better (i.e. 32) */				/*!< Caller(Name) to use on outgoing calls */
 
 	int amaflags;												/*!< amaflags */
 	boolean_t echocancel;											/*!< echocancel phone support */
 	boolean_t silencesuppression;										/*!< Silence Suppression Phone Support */
 	boolean_t meetme;											/*!< Meetme on/off */
 	boolean_t transfer;											/*!< Transfer Phone Support */
+	uint16_t dnd;												/*!< dnd on line */
 	uint16_t dndmode;											/*!< dnd mode: see SCCP_DNDMODE_* */
 
 	SCCP_LIST_HEAD (, sccp_mailbox_t) mailboxes;								/*!< Mailbox Linked List Entry. To check for messages */
@@ -879,7 +872,6 @@ struct sccp_line {
 	char language[SCCP_MAX_LANGUAGE];									/*!< language we use for calls */
 	char accountcode[SCCP_MAX_ACCOUNT_CODE];								/*!< accountcode used in cdr */
 	char musicclass[SCCP_MAX_MUSICCLASS];									/*!< musicclass assigned when getting moh */
-	char parkinglot[SCCP_MAX_CONTEXT];									/*!< parkinglot to use */
 };														/*!< SCCP Line Structure */
 
 /*!
@@ -907,19 +899,15 @@ struct sccp_device {
 	uint32_t skinny_type;											/*!< Model of this Phone sent by the station, devicetype */
 	uint32_t device_features;										/*!< device features (contains protocolversion in 8bit first segement */
 	sccp_devicestate_t state;										/*!< Device State (SCCP_DEVICE_ONHOOK or SCCP_DEVICE_OFFHOOK) */
-	sccp_earlyrtp_t earlyrtp;										/*!< RTP Channel State where to open the RTP Media Stream */
+	sccp_channelstate_t earlyrtp;										/*!< RTP Channel State where to open the RTP Media Stream */
 	sccp_session_t *session;										/*!< Current Session */
 	SCCP_RWLIST_ENTRY (sccp_device_t) list;									/*!< Global Device Linked List */
-	uint16_t keepalive;											/*!< Station Specific Keepalive Timeout */
-	uint16_t keepaliveinterval;										/*!< Currently set Keepalive Timeout */
+	uint16_t keepalive;												/*!< Station Specific Keepalive Timeout */
+	uint16_t keepaliveinterval;											/*!< Currently set Keepalive Timeout */
 	uint8_t protocolversion;										/*!< Skinny Supported Protocol Version */
 	uint8_t inuseprotocolversion;										/*!< Skinny Used Protocol Version */
 	uint16_t registrationState;										/*!< If the device has been fully registered yet */
-#ifndef CS_EXPERIMENTAL
 	boolean_t nat;												/*!< Network Address Translation Support (Boolean, default=on) */
-#else
-	sccp_nat_t nat;												/*!< Network Address Translation Support (Boolean, default=on) */
-#endif
 	boolean_t directrtp;											/*!< Direct RTP Support (Boolean, default=on) */
 
 	sccp_channel_t *active_channel;										/*!< Active SCCP Channel */
@@ -944,7 +932,6 @@ struct sccp_device {
 	uint16_t accessoryused;											/*!< Accessory Used. This are for support of message 0x0073 AccessoryStatusMessage - Protocol v.11 CCM7 -FS */
 	uint16_t accessorystatus;										/*!< Accessory Status */
 	char imageversion[StationMaxImageVersionSize];								/*!< Version to Send to the phone */
-	char loadedimageversion[StationMaxImageVersionSize];							/*!< Loaded version on the phone */
 	char config_type[10];											/*!< Model of this Phone used for setting up features/softkeys/buttons etc. */
 	uint16_t maxstreams;											/*!< Maximum number of Stream supported by the device */
 	int tz_offset;												/*!< Timezone OffSet */
@@ -1020,8 +1007,8 @@ struct sccp_device {
 	boolean_t mwiLight;											/*!< MWI/Light \todo third MWI/light entry in device ? */
 
 	struct {
-		uint16_t newmsgs;										/*!< New Messages */
-		uint16_t oldmsgs;										/*!< Old Messages */
+		uint16_t newmsgs;											/*!< New Messages */
+		uint16_t oldmsgs;											/*!< Old Messages */
 	} voicemailStatistic;											/*!< VoiceMail Statistics */
 
 	/* feature configurations */
@@ -1044,11 +1031,11 @@ struct sccp_device {
 		uint8_t size;											/*!< how many softkeysets are provided by modes */
 		uint32_t activeMask[16];									/*!< enabled softkeys mask */
 	} softKeyConfiguration;											/*!< SoftKeySet configuration */
-
+	
 	struct {
 		btnlist *instance;										/*!< used softkeySet */
 		uint8_t size;											/*!< how many softkeysets are provided by modes */
-	} speeddialButtons;
+	} speeddialButtons;	
 
 	struct {
 		int free;
@@ -1064,6 +1051,7 @@ struct sccp_device {
 		uint32_t transactionID;
 	} dtu_softkey;
 
+
 	boolean_t (*checkACL) (sccp_device_t * device);								/*!< check ACL callback function */
 	sccp_push_result_t (*pushURL) (const sccp_device_t * device, const char *url, uint8_t priority, uint8_t tone);
 	sccp_push_result_t (*pushTextMessage) (const sccp_device_t * device, const char *messageText, const char *from, uint8_t priority, uint8_t tone);
@@ -1072,14 +1060,13 @@ struct sccp_device {
 	void (*setBackgroundImage) (const sccp_device_t * device, const char *url);				/*!< set device background image */
 	void (*displayBackgroundImagePreview) (const sccp_device_t * device, const char *url);			/*!< display background image as preview */
 	void (*setRingTone) (const sccp_device_t * device, const char *url);					/*!< set the default Ringtone */
-	 sccp_dtmfmode_t(*getDtmfMode) (const sccp_device_t * device);
 
 	char *(messageStack[SCCP_MAX_MESSAGESTACK]);								/*!< Message Stack Array */
 #ifndef SCCP_ATOMIC
 	sccp_mutex_t messageStackLock;										/*!< Message Stack Lock */
 #endif
 	sccp_call_statistics_t call_statistics[2];								/*!< Call statistics */
-	sccp_conference_t *conference;										/*!< conference we are part of *//*! \todo to be removed in favor of conference_id */
+	sccp_conference_t *conference;										/*!< conference we are part of // to be removed in favor of conference_id */
 	uint32_t conference_id;											/*!< Conference ID */
 #ifdef CS_SCCP_CONFERENCE
 	boolean_t conferencelist_active;									/*!< ConfList is being displayed on this device */
@@ -1092,10 +1079,9 @@ struct sccp_device {
 #endif
 	uint16_t defaultLineInstance;										/*!< Default Line Instance */
 	char softkeyDefinition[50];										/*!< requested softKey configuration */
-#ifdef CS_EXPERIMENTAL
-	sccp_softKeySetConfiguration_t *softkeyset;
-#endif
-
+	
+//	PBX_ENDPOINT_TYPE *endpoint;
+	
 	boolean_t pendingDelete;										/*!< this bit will tell the scheduler to delete this line when unused */
 	boolean_t pendingUpdate;										/*!< this will contain the updated line struct once reloaded from config to update the line when unused */
 };
@@ -1160,9 +1146,8 @@ struct sccp_channel {
 	int pbx_callid_created;
 #endif
 	uint32_t passthrupartyid;										/*!< Pass Through ID */
-	sccp_channelstate_t state;										/*!< Internal channel state SCCP_CHANNELSTATE_* */
-	sccp_channelstate_t previousChannelState;								/*!< Previous channel state SCCP_CHANNELSTATE_* */
-	sccp_channelstatereason_t channelStateReason;								/*!< Reason the new/current state was set (for example to handle HOLD differently for transfer then normal) */
+	uint16_t state;												/*!< Internal channel state SCCP_CHANNELSTATE_* */
+	uint16_t previousChannelState;										/*!< Previous channel state SCCP_CHANNELSTATE_* */
 	skinny_calltype_t calltype;										/*!< Skinny Call Type as SKINNY_CALLTYPE_* */
 	PBX_CHANNEL_TYPE *owner;										/*!< Asterisk Channel Owner */
 	sccp_line_t *line;											/*!< SCCP Line */
@@ -1205,10 +1190,6 @@ struct sccp_channel {
 	} enbloc;
 
 	struct {
-#ifndef SCCP_ATOMIC
-		sccp_mutex_t lock;
-#endif
-		volatile CAS32_TYPE deny;
 		int digittimeout;										/*!< Schedule for Timeout on Dialing State */
 		int hangup;											/*!< Automatic hangup after invalid/congested indication */
 	} scheduler;
@@ -1218,8 +1199,6 @@ struct sccp_channel {
 	struct {
 		struct sccp_rtp audio;										/*!< Asterisk RTP */
 		struct sccp_rtp video;										/*!< Video RTP session */
-		//struct sccp_rtp text;                                                                         /*!< Video RTP session */
-		uint8_t peer_dtmfmode;
 	} rtp;
 
 	uint16_t ringermode;											/*!< Ringer Mode */
@@ -1233,20 +1212,20 @@ struct sccp_channel {
 
 	sccp_channel_t *parentChannel;										/*!< if we are a cfwd channel, our parent is this */
 
-	sccp_conference_t *conference;										/*!< are we part of a conference? *//*! \todo to be removed instead of conference_id */
+	sccp_conference_t *conference;										/*!< are we part of a conference? // to be removed instead of conference_id */
 	uint32_t conference_id;											/*!< Conference ID (might be safer to use instead of conference) */
 	uint32_t conference_participant_id;									/*!< Conference Participant ID */
 
 	unsigned int maxBitRate;
 	boolean_t peerIsSCCP;											/*!< Indicates that channel-peer is also SCCP */
 	void (*setMicrophone) (sccp_channel_t * channel, boolean_t on);
+//	boolean_t (*hangupRequest) (sccp_channel_t * channel);
 	boolean_t (*hangupRequest) (sccp_channel_t * channel);
 	boolean_t (*isMicrophoneEnabled) (void);
 
 	/* next should be converted to pointers, to reduce size */
 	char musicclass[SCCP_MAX_MUSICCLASS];									/*!< Music Class */
 	sccp_callinfo_t callInfo;
-	sccp_dtmfmode_t dtmfmode;										/*!< DTMF Mode (0 inband - 1 outofband) */
 };														/*!< SCCP Channel Structure */
 
 /*!
@@ -1260,8 +1239,8 @@ struct sccp_global_vars {
 	sccp_mutex_t lock;											/*!< Asterisk: Lock Me Up and Tie me Down */
 
 #if ASTERISK_VERSION_GROUP < 110
-	pthread_t monitor_thread;										/*!< Monitor Thread */
-	sccp_mutex_t monitor_lock;										/*!< Monitor Asterisk Lock */
+	pthread_t monitor_thread;										/*!< Monitor Thread */// ADDED IN 414 -FS
+	sccp_mutex_t monitor_lock;										/*!< Monitor Asterisk Lock */// ADDED IN 414 -FS
 #endif
 
 	sccp_threadpool_t *general_threadpool;									/*!< General Work Threadpool */
@@ -1298,10 +1277,11 @@ struct sccp_global_vars {
 	uint8_t transfer_tone;											/*!< Transfer Tone */
 	uint8_t callwaiting_tone;										/*!< Call Waiting Tone */
 	uint8_t callwaiting_interval;										/*!< Call Waiting Ring Interval */
-	boolean_t dndFeature;											/*!< Do Not Disturb (DND) Mode: \see SCCP_DNDMODE_* */
+	uint8_t dndmode;											/*!< Do Not Disturb (DND) Mode: \see SCCP_DNDMODE_* */
 	uint16_t protocolversion;										/*!< Skinny Protocol Version */
 	boolean_t transfer_on_hangup;										/*!< Complete transfer on hangup */
 
+
 #ifdef CS_MANAGER_EVENTS
 	boolean_t callevents;											/*!< Call Events */
 #endif
@@ -1316,19 +1296,15 @@ struct sccp_global_vars {
 	boolean_t echocancel;											/*!< Echo Canel Support (Boolean, default=on) */
 	boolean_t silencesuppression;										/*!< Silence Suppression Support (Boolean, default=on)  */
 	boolean_t trustphoneip;											/*!< Trust Phone IP Support (Boolean, default=on) */
-	sccp_earlyrtp_t earlyrtp;										/*!< Channel State where to open the rtp media stream */
+	sccp_channelstate_t earlyrtp;										/*!< Channel State where to open the rtp media stream */
 	boolean_t privacy;											/*!< Privacy Support (Length=2) */
 	skinny_lampmode_t mwilamp;										/*!< MWI/Lamp (Length:3) */
 	boolean_t mwioncall;											/*!< MWI On Call Support (Boolean, default=on) */
-	sccp_blindtransferindication_t blindtransferindication;							/*!< Blind Transfer Indication Support (Boolean, default=on = SCCP_BLINDTRANSFER_MOH) */
+	boolean_t blindtransferindication;									/*!< Blind Transfer Indication Support (Boolean, default=on = SCCP_BLINDTRANSFER_MOH) */
 	boolean_t cfwdall;											/*!< Call Forward All Support (Boolean, default=on) */
 	boolean_t cfwdbusy;											/*!< Call Forward on Busy Support (Boolean, default=on) */
 	boolean_t cfwdnoanswer;											/*!< Call Forward on No-Answer Support (Boolean, default=on) */
-#ifndef CS_EXPERIMENTAL
 	boolean_t nat;												/*!< Network Address Translation */
-#else
-	sccp_nat_t nat;												/*!< Network Address Translation */
-#endif
 	boolean_t directrtp;											/*!< Direct RTP */
 	boolean_t useoverlap;											/*!< Overlap Dial Support */
 	sccp_group_t callgroup;											/*!< Call Group */
@@ -1336,7 +1312,7 @@ struct sccp_global_vars {
 	sccp_group_t pickupgroup;										/*!< PickUp Group */
 	boolean_t directed_pickup_modeanswer;									/*!< Directed PickUp Mode Answer (boolean, default" on) */
 #endif
-	sccp_call_answer_order_t callanswerorder;								/*!< Call Answer Order */
+	call_answer_order_t callanswerorder;									/*!< Call Answer Order */
 	boolean_t meetme;											/*!< Meetme on/off */
 	char meetmeopts[SCCP_MAX_CONTEXT];									/*!< Meetme Options to be Used */
 	boolean_t allowAnonymous;										/*!< Allow Anonymous/Guest Devices */
@@ -1387,7 +1363,7 @@ struct sccp_hotline {
 ({															\
 	int _count = 0; 												\
 	int _sched_res = -1; 												\
-	while (id > -1 && (_sched_res = PBX(sched_del)(id)) && ++_count < 10) 						\
+	while (id > -1 && (_sched_res = sccp_sched_del(id)) && ++_count < 10) 						\
 		usleep(1); 												\
 	if (_count == 10) { 												\
 		sccp_log((DEBUGCAT_CORE))(VERBOSE_PREFIX_3 "SCCP: Unable to cancel schedule ID %d.\n", id); 		\
@@ -1404,6 +1380,8 @@ sccp_channel_request_status_t sccp_requestChannel(const char *lineName, skinny_c
 
 int sccp_sched_free(void *ptr);
 
+typedef struct softKeySetConfiguration sccp_softKeySetConfiguration_t;						/*!< SoftKeySet configuration */
+
 /*!
  * \brief SCCP SoftKeySet Configuration Structure
  */
@@ -1411,9 +1389,6 @@ struct softKeySetConfiguration {
 	char name[50];												/*!< Name for this configuration */
 	softkey_modes modes[16];										/*!< SoftKeySet modes, see KEYMODE_ */
 	uint8_t numberOfSoftKeySets;										/*!< Number of SoftKeySets we define */
-#ifdef CS_EXPERIMENTAL
-	sccp_softkeyMap_cb_t *softkeyCbMap;									/*!< Softkey Callback Map, ie handlers */
-#endif
 	unsigned int pendingDelete:1;
 	unsigned int pendingUpdate:1;
 	SCCP_LIST_ENTRY (sccp_softKeySetConfiguration_t) list;							/*!< Next list entry */
@@ -1429,6 +1404,8 @@ boolean_t sccp_postPBX_load(void);
 
 typedef int (*sccp_sched_cb) (const void *data);
 
+int sccp_sched_add(int when, sccp_sched_cb callback, const void *data);
+int sccp_sched_del(int id);
 int sccp_updateExternIp(void);
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -1440,4 +1417,3 @@ int sccp_updateExternIp(void);
 extern const char devstate_db_family[];
 #endif
 #endif														/* __CHAN_SCCP_H */
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
