@@ -1,15 +1,16 @@
+
 /*!
- * \file        ast.c
- * \brief       SCCP PBX Asterisk Wrapper Class
- * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
- * \note        Reworked, but based on chan_sccp code.
- *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *              Modified by Jan Czmok and Julien Goodwin
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \file	ast.c
+ * \brief	SCCP PBX Asterisk Wrapper Class
+ * \author	Diederik de Groot <ddegroot [at] users.sourceforge.net>
+ * \note	Reworked, but based on chan_sccp code.
+ * 		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ * 		Modified by Jan Czmok and Julien Goodwin
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $  
+ * $Date: 2010-10-23 20:04:30 +0200 (Sat, 23 Oct 2010) $
+ * $Revision: 2044 $  
  */
 #include <config.h>
 #include "../../common.h"
@@ -18,9 +19,8 @@
 #include "../../sccp_utils.h"
 #include "../../sccp_indicate.h"
 #include "../../sccp_socket.h"
-#include "../../sccp_pbx.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
+SCCP_FILE_VERSION(__FILE__, "$Revision: 2269 $")
 
 /*
  * \brief itterate through locked pbx channels
@@ -53,7 +53,7 @@ PBX_CHANNEL_TYPE *pbx_channel_walk_locked(PBX_CHANNEL_TYPE * target)
 
 	if (tmp) {
 		res = tmp;
-		tmp = pbx_channel_unref(tmp);
+		tmp = ast_channel_unref(tmp);
 		ast_channel_lock(res);
 	}
 	ast_channel_iterator_destroy(iter);
@@ -87,7 +87,7 @@ PBX_CHANNEL_TYPE *pbx_channel_search_locked(int (*is_match) (PBX_CHANNEL_TYPE *,
 		return NULL;
 	}
 
-	for (; iter && (tmp = ast_channel_iterator_next(iter)); tmp = pbx_channel_unref(tmp)) {
+	for (; iter && (tmp = ast_channel_iterator_next(iter)); tmp = ast_channel_unref(tmp)) {
 		pbx_channel_lock(tmp);
 		if (is_match(tmp, data)) {
 			matched = TRUE;
@@ -102,7 +102,7 @@ PBX_CHANNEL_TYPE *pbx_channel_search_locked(int (*is_match) (PBX_CHANNEL_TYPE *,
 
 	if (matched) {
 		pbx_channel = tmp;
-		tmp = pbx_channel_unref(tmp);
+		tmp = ast_channel_unref(tmp);
 		return pbx_channel;
 	} else {
 		return NULL;
@@ -429,107 +429,102 @@ sccp_channel_t *get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx_c
 	}
 }
 
-//static boolean_t sccp_wrapper_asterisk_nullHangup(sccp_channel_t *channel)
-//{
-//      return FALSE;
-//}
+#ifdef CS_EXPERIMENTAL
+boolean_t sccp_wrapper_asterisk_dummyHangup(sccp_channel_t *channel)
+{
+	pbx_log(LOG_NOTICE, "%s: hangup request, replaced by dummy. Hangup not performed\n", channel->designator);
+	return FALSE;
+}
 
-static boolean_t sccp_wrapper_asterisk_carefullHangup(sccp_channel_t * c)
+boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t *channel)
 {
 	boolean_t res = FALSE;
+	PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
 
-	if (!c || !c->owner) {
+	channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
+	if (!ast_check_hangup(pbx_channel)) {							/* if channel is not already been hungup */
+		res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
+	} else {
+		pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_requestQueueHangup) Already Hungup\n", channel->designator);
+	}
+	return res;
+}
+
+// carefull version of sccp_wrapper_asterisk_requestHangup, when ast_pbx_start has been requested.
+boolean_t sccp_wrapper_asterisk_requestHangup_PBXStarting(sccp_channel_t *channel)
+{
+	PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
+	channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
+	if (!pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) && !pbx_channel_pbx(pbx_channel) && !pbx_check_hangup(pbx_channel)) {
+		ast_hangup(channel->owner);
+	} else {
+		sccp_wrapper_asterisk_requestQueueHangup(channel);
+	}
+	return TRUE;
+}
+
+boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t *channel)
+{
+	channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
+	ast_hangup(channel->owner);
+	return TRUE;
+}
+#else
+int sccp_wrapper_asterisk_requestHangup(PBX_CHANNEL_TYPE * ast_channel)
+{
+	if (!ast_channel) {
+		pbx_log(LOG_NOTICE, "channel to hangup is NULL\n");
 		return FALSE;
 	}
-	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
 
-	if (channel && channel->owner) {
-		PBX_CHANNEL_TYPE *pbx_channel = pbx_channel_ref(channel->owner);
-
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
-
-		/* let's wait for a bit, for the dust to settle */
-		sched_yield();
-		pbx_safe_sleep(pbx_channel, 1000);
-
-		/* recheck everything before going forward */
-		pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) processing hangup request, using carefull version. Standby.\n", pbx_channel_name(pbx_channel));
-		if (!pbx_check_hangup(pbx_channel)) {
-			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Channel still active.\n", pbx_channel_name(pbx_channel));
-			if (pbx_channel_pbx(pbx_channel) || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING) || pbx_channel_state(pbx_channel) == AST_STATE_UP) {
-				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has PBX -> ast_queue_hangup.\n", pbx_channel_name(pbx_channel));
-				res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
-			} else {
-				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has no PBX -> ast_hangup.\n", pbx_channel_name(pbx_channel));
-				ast_hangup(pbx_channel);
-				res = TRUE;
-			}
-		} else {
-			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Already Hungup. Forcing SCCP Remove Call.\n", pbx_channel_name(pbx_channel));
-			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
-			if (d) {
-				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
-			}
-			res = TRUE;
-		}
-		pbx_channel_unref(pbx_channel);
+	if ((pbx_channel_softhangup(ast_channel) & AST_SOFTHANGUP_APPUNLOAD) != 0) {
+		pbx_channel_set_hangupcause(ast_channel, AST_CAUSE_CHANNEL_UNACCEPTABLE);
+		ast_softhangup(ast_channel, AST_SOFTHANGUP_APPUNLOAD);
+		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: send softhangup appunload\n", pbx_channel_name(ast_channel));
+		return TRUE;
 	}
-	return res;
-}
 
-boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t * c)
-{
-	boolean_t res = FALSE;
-	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
+	sccp_channel_t *sccp_channel = get_sccp_channel_from_pbx_channel(ast_channel);
 
-	if (channel) {
-		PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
+	sccp_log((DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "hangup %s: hasPbx %s; ast state: %s, sccp state: %s, blocking: %s, already being hungup: %s, hangupcause: %d\n",
+				      pbx_channel_name(ast_channel), pbx_channel_pbx(ast_channel) ? "yes" : "no", pbx_state2str(pbx_channel_state(ast_channel)), sccp_channel ? sccp_indicate2str(sccp_channel->state) : "--", pbx_test_flag(pbx_channel_flags(ast_channel), AST_FLAG_BLOCKING) ? "yes" : "no", pbx_channel_softhangup(ast_channel) ? "yes" : "no", pbx_channel_hangupcause(ast_channel)
+	    );
 
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
+	if (pbx_test_flag(pbx_channel_flags(ast_channel), AST_FLAG_BLOCKING)) {
+		// wait for blocker before issuing softhangup
+		int tries = 0;
 
-		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
-		if (!pbx_check_hangup(pbx_channel)) {								/* if channel is not already been hungup */
-			res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
+		while (!pbx_channel_blocker(ast_channel) && tries < 50) {
+			sccp_log((DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "SCCP: (requestHangup) Blocker set but no blocker found yet, waiting...!\n");
+			usleep(50);
+			tries++;
+		}
+		sccp_log((DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: send ast_softhangup_nolock (blocker: %s)\n", pbx_channel_name(ast_channel), pbx_channel_blockproc(ast_channel));
+		ast_softhangup_nolock(ast_channel, AST_SOFTHANGUP_DEV);
+	} else if (AST_STATE_UP == pbx_channel_state(ast_channel) || pbx_channel_pbx(ast_channel)) {
+		sccp_log((DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: send ast_queue_hangup\n", pbx_channel_name(ast_channel));
+#if ASTERISK_VERSION_NUMBER < 10601
+		pbx_channel_setwhentohangup_tv(ast_channel, 0);
+#else
+		pbx_channel_setwhentohangup_tv(ast_channel, ast_tvnow());
+#endif
+		ast_queue_hangup(ast_channel);
+	} else {
+		sccp_log((DEBUGCAT_CHANNEL)) (VERBOSE_PREFIX_3 "%s: send hard ast_hangup\n", pbx_channel_name(ast_channel));
+		if (!pbx_channel_pbx(ast_channel)) {
+			ast_channel_lock(ast_channel);
+			ast_indicate(ast_channel, -1);
+			ast_hangup(ast_channel);
+			ast_channel_unlock(ast_channel);
 		} else {
-			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_requestQueueHangup) Already Hungup\n", channel->designator);
-			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
-			if (d) {
-				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
-			}
+			ast_queue_hangup(ast_channel);
 		}
 	}
-	return res;
-}
 
-boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t * c)
-{
-	boolean_t res = FALSE;
-	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
-
-	if (channel) {
-		PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
-
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
-
-		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
-		if (!pbx_check_hangup(pbx_channel)) {
-			if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING)) {
-				return sccp_wrapper_asterisk_requestQueueHangup(channel);
-			}
-			ast_hangup(pbx_channel);
-			res = TRUE;
-		} else {
-			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
-			if (d) {
-				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
-			}
-		}
-	}
-	return res;
+	sccp_channel = sccp_channel ? sccp_channel_release(sccp_channel) : NULL;
+	return TRUE;
 }
+#endif
 
 int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcname, char *args, const char *value)
 {
@@ -552,30 +547,6 @@ int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcna
 
 		} else if (!strcasecmp(args, "codec")) {
 			res = sccp_channel_setPreferredCodec(c, value);
-		} else if (!strcasecmp(args, "CallingParty")) {
-			char *num, *name;
-
-			pbx_callerid_parse((char *) value, &name, &num);
-			sccp_channel_set_callingparty(c, name, num);
-			sccp_channel_display_callInfo(c);
-		} else if (!strcasecmp(args, "CalledParty")) {
-			char *num, *name;
-
-			pbx_callerid_parse((char *) value, &name, &num);
-			sccp_channel_set_calledparty(c, name, num);
-			sccp_channel_display_callInfo(c);
-		} else if (!strcasecmp(args, "OriginalCallingParty")) {
-			char *num, *name;
-
-			pbx_callerid_parse((char *) value, &name, &num);
-			sccp_channel_set_originalCallingparty(c, name, num);
-			sccp_channel_display_callInfo(c);
-		} else if (!strcasecmp(args, "OriginalCalledParty")) {
-			char *num, *name;
-
-			pbx_callerid_parse((char *) value, &name, &num);
-			sccp_channel_set_originalCalledparty(c, name, num);
-			sccp_channel_display_callInfo(c);
 		} else if (!strcasecmp(args, "microphone")) {
 			if (!value || sccp_strlen_zero(value) || !sccp_true(value)) {
 				c->setMicrophone(c, FALSE);
@@ -687,8 +658,6 @@ void sccp_asterisk_redirectedUpdate(sccp_channel_t * channel, const void *data,
 	sccp_copy_string(channel->callInfo.lastRedirectingPartyNumber, ast->cid.cid_rdnis ? ast->cid.cid_rdnis : "", sizeof(channel->callInfo.lastRedirectingPartyNumber));
 	channel->callInfo.lastRedirectingParty_valid = 1;
 #endif
-	channel->callInfo.originalCdpnRedirectReason = channel->callInfo.lastRedirectingReason;
-	channel->callInfo.lastRedirectingReason = 4;								// need to figure out these codes
 
 	sccp_channel_send_callinfo2(channel);
 }
@@ -758,17 +727,12 @@ void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const ch
  *
  * \called_from_asterisk
  */
-int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *funcname, char *preparse, char *buf, size_t buflen)
+int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *funcname, char *args, char *buf, size_t buflen)
 {
 	sccp_channel_t *c = NULL;
 	sccp_device_t *d = NULL;
 	int res = 0;
 
-	char *parse = sccp_strdupa(preparse);
-
-	AST_DECLARE_APP_ARGS(args, AST_APP_ARG(param); AST_APP_ARG(type); AST_APP_ARG(field););
-	AST_STANDARD_APP_ARGS(args, parse);
-
 	if (!ast || !CS_AST_CHANNEL_PVT_IS_SCCP(ast)) {
 		pbx_log(LOG_ERROR, "This function requires a valid SCCP channel\n");
 		return -1;
@@ -776,133 +740,14 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 
 	if ((c = get_sccp_channel_from_pbx_channel(ast))) {
 		if ((d = sccp_channel_getDevice_retained(c))) {
-			if (!strcasecmp(args.param, "peerip")) {
-				sccp_copy_string(buf, sccp_socket_stringify(&d->session->sin), buflen);
-			} else if (!strcasecmp(args.param, "recvip")) {
+			if (!strcasecmp(args, "peerip")) {
+                                sccp_copy_string(buf, sccp_socket_stringify(&d->session->sin), buflen);
+			} else if (!strcasecmp(args, "recvip")) {
 				ast_copy_string(buf, sccp_socket_stringify(&d->session->sin), buflen);
-			} else if (!strcasecmp(args.param, "useragent")) {
-				sccp_copy_string(buf, skinny_devicetype2str(d->skinny_type), buflen);
-			} else if (!strcasecmp(args.param, "from")) {
+			} else if (!strcasecmp(args, "useragent")) {
+				sccp_copy_string(buf, devicetype2str(d->skinny_type), buflen);
+			} else if (!strcasecmp(args, "from")) {
 				sccp_copy_string(buf, (char *) d->id, buflen);
-#if ASTERISK_VERSION_GROUP >= 108
-			} else if (!strcasecmp(args.param, "rtpqos")) {
-				PBX_RTP_TYPE *rtp = NULL;
-
-				if (sccp_strlen_zero(args.type)) {
-					args.type = "audio";
-				}
-
-				if (sccp_strcaseequals(args.type, "audio")) {
-					rtp = c->rtp.audio.rtp;
-				} else if (sccp_strcaseequals(args.type, "video")) {
-					rtp = c->rtp.video.rtp;
-				/*
-				} else if (sccp_strcaseequals(args.type, "text")) {
-					rtp = c->rtp.text.rtp;
-				*/
-				} else {
-					return -1;
-				}
-
-				if (sccp_strlen_zero(args.field) || sccp_strcaseequals(args.field, "all")) {
-					char quality_buf[256 /*AST_MAX_USER_FIELD */ ];
-
-					if (!ast_rtp_instance_get_quality(rtp, AST_RTP_INSTANCE_STAT_FIELD_QUALITY, quality_buf, sizeof(quality_buf))) {
-						return -1;
-					}
-
-					sccp_copy_string(buf, quality_buf, buflen);
-					return res;
-				} else {
-					struct ast_rtp_instance_stats stats;
-					int i;
-					struct {
-						const char *name;
-						enum { INT, DBL } type;
-						union {
-							unsigned int *i4;
-							double *d8;
-						};
-					} lookup[] = {
-						{
-							"txcount", INT, {
-						.i4 = &stats.txcount,},}, {
-							"rxcount", INT, {
-						.i4 = &stats.rxcount,},}, {
-							"txjitter", DBL, {
-						.d8 = &stats.txjitter,},}, {
-							"rxjitter", DBL, {
-						.d8 = &stats.rxjitter,},}, {
-							"remote_maxjitter", DBL, {
-						.d8 = &stats.remote_maxjitter,},}, {
-							"remote_minjitter", DBL, {
-						.d8 = &stats.remote_minjitter,},}, {
-							"remote_normdevjitter", DBL, {
-						.d8 = &stats.remote_normdevjitter,},}, {
-							"remote_stdevjitter", DBL, {
-						.d8 = &stats.remote_stdevjitter,},}, {
-							"local_maxjitter", DBL, {
-						.d8 = &stats.local_maxjitter,},}, {
-							"local_minjitter", DBL, {
-						.d8 = &stats.local_minjitter,},}, {
-							"local_normdevjitter", DBL, {
-						.d8 = &stats.local_normdevjitter,},}, {
-							"local_stdevjitter", DBL, {
-						.d8 = &stats.local_stdevjitter,},}, {
-							"txploss", INT, {
-						.i4 = &stats.txploss,},}, {
-							"rxploss", INT, {
-						.i4 = &stats.rxploss,},}, {
-							"remote_maxrxploss", DBL, {
-						.d8 = &stats.remote_maxrxploss,},}, {
-							"remote_minrxploss", DBL, {
-						.d8 = &stats.remote_minrxploss,},}, {
-							"remote_normdevrxploss", DBL, {
-						.d8 = &stats.remote_normdevrxploss,},}, {
-							"remote_stdevrxploss", DBL, {
-						.d8 = &stats.remote_stdevrxploss,},}, {
-							"local_maxrxploss", DBL, {
-						.d8 = &stats.local_maxrxploss,},}, {
-							"local_minrxploss", DBL, {
-						.d8 = &stats.local_minrxploss,},}, {
-							"local_normdevrxploss", DBL, {
-						.d8 = &stats.local_normdevrxploss,},}, {
-							"local_stdevrxploss", DBL, {
-						.d8 = &stats.local_stdevrxploss,},}, {
-							"rtt", DBL, {
-						.d8 = &stats.rtt,},}, {
-							"maxrtt", DBL, {
-						.d8 = &stats.maxrtt,},}, {
-							"minrtt", DBL, {
-						.d8 = &stats.minrtt,},}, {
-							"normdevrtt", DBL, {
-						.d8 = &stats.normdevrtt,},}, {
-							"stdevrtt", DBL, {
-						.d8 = &stats.stdevrtt,},}, {
-							"local_ssrc", INT, {
-						.i4 = &stats.local_ssrc,},}, {
-							"remote_ssrc", INT, {
-						.i4 = &stats.remote_ssrc,},}, {
-					NULL,},};
-
-					if (ast_rtp_instance_get_stats(rtp, &stats, AST_RTP_INSTANCE_STAT_ALL)) {
-						return -1;
-					}
-
-					for (i = 0; !sccp_strlen_zero(lookup[i].name); i++) {
-						if (sccp_strcaseequals(args.field, lookup[i].name)) {
-							if (lookup[i].type == INT) {
-								snprintf(buf, buflen, "%u", *lookup[i].i4);
-							} else {
-								snprintf(buf, buflen, "%f", *lookup[i].d8);
-							}
-							return 0;
-						}
-					}
-					pbx_log(LOG_WARNING, "SCCP: (sccp_wrapper_asterisk_channel_read) Unrecognized argument '%s' to %s\n", preparse, funcname);
-					return -1;
-				}
-#endif
 			} else {
 				res = -1;
 			}
@@ -921,11 +766,9 @@ boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel)
 {
 #if ASTERISK_VERSION_GROUP >= 112
 	char *featexten;
-
-	if (PBX(getFeatureExtension) (channel, &featexten)) {
+	if (PBX(getFeatureExtension)(channel, &featexten)) {
 		struct ast_frame f = { AST_FRAME_DTMF, };
 		int j;
-
 		f.len = 100;
 		for (j = 0; j < strlen(featexten); j++) {
 			f.subclass.integer = featexten[j];
@@ -933,19 +776,18 @@ boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel)
 		}
 		sccp_free(featexten);
 		return TRUE;
-	}
+        }
 #else
 	struct ast_call_feature *feature = ast_find_call_feature("automon");
-
 	if (feature) {
 		feature->operation(channel->owner, channel->owner, NULL, "monitor button", 0, NULL);
-		/*
-		   if (pbx_channel_monitor(channel->owner)) {
-		   return TRUE;
-		   } else {
-		   return FALSE;
-		   }
-		 */
+/*
+		if (pbx_channel_monitor(channel->owner)) {
+			return TRUE;
+		} else {
+			return FALSE;
+		}
+*/
 		return TRUE;
 	}
 #endif
@@ -954,51 +796,8 @@ boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel)
 
 }
 
-#if !defined(AST_DEFAULT_EMULATE_DTMF_DURATION)
-#define AST_DEFAULT_EMULATE_DTMF_DURATION 100
-#endif
-#if ASTERISK_VERSION_GROUP > 106
-int sccp_wrapper_sendDigits(const sccp_channel_t * channel, const char *digits)
-{
-	if (!channel || !channel->owner) {
-		pbx_log(LOG_WARNING, "No channel to send digits to\n");
-		return 0;
-	}
-	if (!digits || sccp_strlen_zero(digits)) {
-		pbx_log(LOG_WARNING, "No digits to send\n");
-		return 0;
-	}
-	//ast_channel_undefer_dtmf(channel->owner);
-	PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
-	int i;
-	PBX_FRAME_TYPE f = ast_null_frame;
 
-	sccp_log((DEBUGCAT_HIGH)) (VERBOSE_PREFIX_3 "%s: Sending digits '%s'\n", (char *) channel->currentDeviceId, digits);
-	// We don't just call sccp_pbx_senddigit due to potential overhead, and issues with locking
-	f.src = "SCCP";
-	for (i = 0; digits[i] != '\0'; i++) {
-		sccp_log((DEBUGCAT_HIGH)) (VERBOSE_PREFIX_3 "%s: Sending digit %c\n", (char *) channel->currentDeviceId, digits[i]);
-
-		f.frametype = AST_FRAME_DTMF_END;								// Sending only the dmtf will force asterisk to start with DTMF_BEGIN and schedule the DTMF_END
-		f.subclass.integer = digits[i];
-		// f.len = SCCP_MIN_DTMF_DURATION;
-		f.len = AST_DEFAULT_EMULATE_DTMF_DURATION;
-		f.src = "SEND DIGIT";
-		ast_queue_frame(pbx_channel, &f);
-	}
-	return 1;
-}
-
-int sccp_wrapper_sendDigit(const sccp_channel_t * channel, const char digit)
-{
-	const char digits[] = { digit, '\0' };
-	sccp_log((DEBUGCAT_HIGH)) (VERBOSE_PREFIX_3 "%s: got a single digit '%c' -> '%s'\n", channel->currentDeviceId, digit, digits);
-	return sccp_wrapper_sendDigits(channel, digits);
-}
-#endif
-
-static void *sccp_asterisk_doPickupThread(void *data)
-{
+static void *sccp_asterisk_doPickupThread(void *data) {
 	PBX_CHANNEL_TYPE *pbx_channel = data;
 
 	if (ast_pickup_call(pbx_channel)) {
@@ -1012,13 +811,11 @@ static void *sccp_asterisk_doPickupThread(void *data)
 	return NULL;
 }
 
-static int sccp_asterisk_doPickup(PBX_CHANNEL_TYPE * pbx_channel)
-{
+static int sccp_asterisk_doPickup(PBX_CHANNEL_TYPE *pbx_channel) {
 	pthread_t threadid;
 
-	if (!pbx_channel || !(pbx_channel_ref(pbx_channel) > 0)) {
-		return FALSE;
-	}
+
+	pbx_channel_ref(pbx_channel);
 	if (ast_pthread_create_detached_background(&threadid, NULL, sccp_asterisk_doPickupThread, pbx_channel)) {
 		pbx_log(LOG_ERROR, "Unable to start Group pickup thread on channel %s\n", pbx_channel_name(pbx_channel));
 		pbx_channel_unref(pbx_channel);
@@ -1028,8 +825,8 @@ static int sccp_asterisk_doPickup(PBX_CHANNEL_TYPE * pbx_channel)
 	return TRUE;
 }
 
-enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
-{
+#ifdef CS_EXPERIMENTAL
+enum ast_pbx_result pbx_pbx_start (PBX_CHANNEL_TYPE *pbx_channel) {
 	enum ast_pbx_result res = AST_PBX_FAILED;
 	sccp_channel_t *channel = NULL;
 
@@ -1037,20 +834,18 @@ enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
 		pbx_log(LOG_ERROR, "SCCP: (pbx_pbx_start) called without pbx channel\n");
 		return res;
 	}
-
-	if ((channel = get_sccp_channel_from_pbx_channel(pbx_channel))) {
+	
+	if((channel = get_sccp_channel_from_pbx_channel(pbx_channel))){
 		ast_channel_lock(pbx_channel);
 #if ASTERISK_VERSION_GROUP >= 111
 		struct ast_callid *callid = NULL;
-
 		channel->pbx_callid_created = ast_callid_threadstorage_auto(&callid);
 		ast_channel_callid_set(pbx_channel, callid);
-#endif
+#endif		
 		// check if the pickup extension was entered
-		const char *dialedNumber = PBX(getChannelExten) (channel);
-		char *pickupexten;
-
-		if (PBX(getPickupExtension) (channel, &pickupexten) && sccp_strequals(dialedNumber, pickupexten)) {
+		const char *dialedNumber = PBX(getChannelExten)(channel);
+		char *pickupexten = "";
+		if (PBX(getPickupExtension)(channel, &pickupexten) && sccp_strequals(dialedNumber, pickupexten)) {
 			if (sccp_asterisk_doPickup(pbx_channel)) {
 				res = AST_PBX_SUCCESS;
 			}
@@ -1059,23 +854,23 @@ enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
 			sccp_free(pickupexten);
 			goto EXIT;
 		}
-		// channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
-		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
-		res = ast_pbx_start(pbx_channel);								// starting ast_pbx_start with a locked ast_channel so we know exactly where we end up when/if the __ast_pbx_run get started
-		if (res == 0) {											// thread started successfully
-			do {											// wait for thread to become ready
+//		channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
+		channel->hangupRequest = sccp_wrapper_asterisk_requestHangup_PBXStarting;
+		res = ast_pbx_start(pbx_channel);			// starting ast_pbx_start with a locked ast_channel so we know exactly where we end up when/if the __ast_pbx_run get started
+		if (res == 0) {						// thread started successfully
+			do {						// wait for thread to become ready
 				pbx_safe_sleep(pbx_channel, 10);
 			} while (!pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) && !pbx_channel_pbx(pbx_channel) && pbx_check_hangup(pbx_channel));
-
+			
 			/* test if __ast_pbx_run got started correctly and if the channel has not already been hungup */
-			// if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) && pbx_channel_pbx(pbx_channel) && !pbx_check_hangup(pbx_channel)) {
-			if (pbx_channel_pbx(pbx_channel) && !pbx_check_hangup(pbx_channel)) {
-				sccp_log(DEBUGCAT_PBX) (VERBOSE_PREFIX_3 "%s: (pbx_pbx_start) autoloop has started, set requestHangup = requestQueueHangup\n", channel->designator);
+			if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) && pbx_channel_pbx(pbx_channel) && !pbx_check_hangup(pbx_channel)) {
+				sccp_log(DEBUGCAT_PBX)(VERBOSE_PREFIX_3 "%s: (pbx_pbx_start) autoloop has started, set requestHangup = requestQueueHangup\n", channel->designator);
 				channel->hangupRequest = sccp_wrapper_asterisk_requestQueueHangup;
 			} else {
-				pbx_log(LOG_NOTICE, "%s: (pbx_pbx_start) pbx_pbx_start thread is not running anymore, carefullHangup should remain. This channel will be hungup/being hungup soon\n", channel->designator);
-				//pbx_log(LOG_NOTICE, "%s: (pbx_pbx_start) Reason: autoloop: %s, pbx_start: %s, already hungup: %s\n", channel->designator, pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_IN_AUTOLOOP) ? "TRUE" : "FALSE", pbx_channel_pbx(pbx_channel) ? "TRUE" : "FALSE", !pbx_check_hangup(pbx_channel) ? "TRUE" : "FALSE");
-				res = AST_PBX_FAILED;
+				pbx_log(LOG_NOTICE, "%s: (pbx_pbx_start) autoloop is not running anymore, dummyHangup should remain. Will already be hungup/being hungup\n", channel->designator);
+				if (!pbx_check_hangup(pbx_channel)) {
+					ast_hangup(pbx_channel);
+				}
 			}
 		}
 		ast_channel_unlock(pbx_channel);
@@ -1084,5 +879,36 @@ enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
 EXIT:
 	return res;
 }
+#else
+enum ast_pbx_result pbx_pbx_start (PBX_CHANNEL_TYPE *pbx_channel) {
+	enum ast_pbx_result res = AST_PBX_FAILED;
+	sccp_channel_t *channel = NULL;
 
+	if (!pbx_channel) {
+		pbx_log(LOG_ERROR, "SCCP: (pbx_pbx_start) called without pbx channel\n");
+		return res;
+	}
+	
+	if((channel = get_sccp_channel_from_pbx_channel(pbx_channel))){
+		ast_channel_lock(pbx_channel);
+		const char *dialedNumber = PBX(getChannelExten)(channel);
+		char *pickupexten = "";
+		if (PBX(getPickupExtension)(channel, &pickupexten) && sccp_strequals(dialedNumber, pickupexten)) {
+			if (sccp_asterisk_doPickup(pbx_channel)) {
+				res = AST_PBX_SUCCESS;
+			}
+			ast_channel_unlock(pbx_channel);
+			channel = sccp_channel_release(channel);
+			sccp_free(pickupexten);
+			goto EXIT;
+		}
+		ast_channel_unlock(pbx_channel);
+		channel = sccp_channel_release(channel);
+	}
+	res = ast_pbx_start(pbx_channel);
+EXIT:
+	return res;
+}
+#endif
+ 
 // kate: indent-width 4; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off;
