@@ -1,15 +1,15 @@
 /*!
- * \file        ast.h
+ * \file	ast.h
  * \brief       SCCP PBX Asterisk Header
- * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
- * \note        Reworked, but based on chan_sccp code.
- *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *              Modified by Jan Czmok and Julien Goodwin
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \author	Diederik de Groot <ddegroot [at] users.sourceforge.net>
+ * \note	Reworked, but based on chan_sccp code.
+ *		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *		Modified by Jan Czmok and Julien Goodwin
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $  
+ * $Date: 2010-10-23 20:04:30 +0200 (Sat, 23 Oct 2010) $
+ * $Revision: 2044 $  
  */
 #ifndef __SCCP_PBX_WRAPPER_H
 #define __SCCP_PBX_WRAPPER_H
@@ -100,7 +100,7 @@
 #	include "ast112.h"
 #endif
 #ifdef ASTERISK_CONF_1_13
-#	include "ast113.h"
+#	include "astTrunk.h"
 #endif
 #include <asterisk/file.h>
 
@@ -127,23 +127,15 @@ static const struct skinny2pbx_codec_map {
 	{SKINNY_CODEC_G711_ALAW_56K,	AST_FORMAT_ALAW},
 	{SKINNY_CODEC_G711_ULAW_64K,	AST_FORMAT_ULAW},
 	{SKINNY_CODEC_G711_ULAW_56K,	AST_FORMAT_ULAW},
+	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723_1},
+	{SKINNY_CODEC_G729,		AST_FORMAT_G729A},
+	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729A},
 	{SKINNY_CODEC_GSM,		AST_FORMAT_GSM},
 	{SKINNY_CODEC_H261,		AST_FORMAT_H261},
 	{SKINNY_CODEC_H263,		AST_FORMAT_H263},
-	{SKINNY_CODEC_T120,		AST_FORMAT_T140},
-#    if ASTERISK_VERSION_GROUP >= 113
-	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723},
-	{SKINNY_CODEC_WIDEBAND_256K,	AST_FORMAT_SLIN16},
-	{SKINNY_CODEC_G729,		AST_FORMAT_G729},
-	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729},
-	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263P},
-#    else
-	{SKINNY_CODEC_G723_1,		AST_FORMAT_G723_1},
-	{SKINNY_CODEC_WIDEBAND_256K,	AST_FORMAT_SLINEAR16},
-	{SKINNY_CODEC_G729,		AST_FORMAT_G729A},
-	{SKINNY_CODEC_G729_A,		AST_FORMAT_G729A},
 	{SKINNY_CODEC_H263P,	 	AST_FORMAT_H263_PLUS},
-#endif
+	{SKINNY_CODEC_T120,		AST_FORMAT_T140},
+	{SKINNY_CODEC_WIDEBAND_256K,	AST_FORMAT_SLINEAR16},
 #    if ASTERISK_VERSION_NUMBER >= 10400
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726_AAL2},
 	{SKINNY_CODEC_G726_32K,		AST_FORMAT_G726},
@@ -363,8 +355,12 @@ sccp_channel_t *__get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx
 sccp_channel_t *get_sccp_channel_from_pbx_channel(const PBX_CHANNEL_TYPE * pbx_channel);
 #endif
 int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcname, char *args, const char *value);
-boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t * channel);
-boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t * channel);
+#ifdef CS_EXPERIMENTAL
+boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t *channel);
+boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t *channel);
+#else
+int sccp_wrapper_asterisk_requestHangup(PBX_CHANNEL_TYPE * pbx_channel);
+#endif
 
 /***** database *****/
 boolean_t sccp_asterisk_addToDatabase(const char *family, const char *key, const char *value);
@@ -380,11 +376,5 @@ void sccp_asterisk_redirectedUpdate(sccp_channel_t * channel, const void *data,
 void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const char *fromNumber, const char *fromName, const char *toNumber, const char *toName, uint8_t reason);
 int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *funcname, char *args, char *buf, size_t buflen);
 boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel);
-
-#if ASTERISK_VERSION_GROUP > 106
-int sccp_wrapper_sendDigits(const sccp_channel_t * channel, const char *digits);
-int sccp_wrapper_sendDigit(const sccp_channel_t * channel, const char digit);
+enum ast_pbx_result pbx_pbx_start (struct ast_channel *ast);
 #endif
-enum ast_pbx_result pbx_pbx_start(struct ast_channel *ast);
-#endif
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
