@@ -1,15 +1,15 @@
 /*!
- * \file        sccp_line.c
- * \brief       SCCP Line
- * \author      Sergio Chersovani <mlists [at] c-net.it>
- * \note        Reworked, but based on chan_sccp code.
- *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *              Modified by Jan Czmok and Julien Goodwin
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \file	sccp_line.c
+ * \brief	SCCP Line
+ * \author	Sergio Chersovani <mlists [at] c-net.it>
+ * \note	Reworked, but based on chan_sccp code.
+ * 		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ * 		Modified by Jan Czmok and Julien Goodwin
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-06-12 20:42:28 +0200 (Fri, 12 Jun 2015) $
- * $Revision: 6122 $
+ * $Date: 2014-04-12 17:21:45 +0200 (Sat, 12 Apr 2014) $
+ * $Revision: 5341 $
  */
 
 #include <config.h>
@@ -22,7 +22,7 @@
 #include "sccp_config.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 6122 $");
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5341 $")
 static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptionId, int onoff);
 int __sccp_line_destroy(const void *ptr);
 int __sccp_lineDevice_destroy(const void *ptr);
@@ -76,39 +76,19 @@ void sccp_line_pre_reload(void)
  */
 void sccp_line_post_reload(void)
 {
-	sccp_line_t *line;
+	sccp_line_t *l;
+	sccp_linedevices_t *linedevice;
 
-	SCCP_RWLIST_TRAVERSE_SAFE_BEGIN(&GLOB(lines), line, list) {
-		if (!line->pendingDelete && !line->pendingUpdate) {
+	SCCP_RWLIST_TRAVERSE_SAFE_BEGIN(&GLOB(lines), l, list) {
+		if (!l->pendingDelete && !l->pendingUpdate) {
 			continue;
 		}
-		AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
-
-		if (l) {
-			// existing lines
-			sccp_linedevices_t *linedevice;
+		if ((l = sccp_line_retain(l))) {
 			SCCP_LIST_LOCK(&l->devices);
 			SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
 				linedevice->device->pendingUpdate = 1;
 			}
 			SCCP_LIST_UNLOCK(&l->devices);
-			
-			if (l->pendingUpdate) {
-				// new line
-				sccp_device_t *d = NULL;
-				sccp_buttonconfig_t *buttonconfig;
-				SCCP_RWLIST_RDLOCK(&GLOB(devices));
-				SCCP_LIST_TRAVERSE(&GLOB(devices), d, list) {
-					SCCP_LIST_LOCK(&d->buttonconfig);
-					SCCP_LIST_TRAVERSE(&d->buttonconfig, buttonconfig, list) {
-						if (buttonconfig->type == LINE && !sccp_strlen_zero(buttonconfig->button.line.name) && !sccp_strequals(line->name, buttonconfig->button.line.name) ) {
-							d->pendingUpdate = TRUE;
-						}
-					}
-					SCCP_LIST_UNLOCK(&d->buttonconfig);
-				}
-				SCCP_RWLIST_UNLOCK(&GLOB(devices));
-			}
 
 			if (l->pendingDelete) {
 				sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Deleting Line (post_reload)\n", l->name);
@@ -117,6 +97,7 @@ void sccp_line_post_reload(void)
 				sccp_log((DEBUGCAT_CONFIG + DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: Cleaning Line (post_reload)\n", l->name);
 				sccp_line_clean(l, FALSE);
 			}
+			l = sccp_line_release(l);
 		}
 	}
 	SCCP_RWLIST_TRAVERSE_SAFE_END;
@@ -135,13 +116,13 @@ void sccp_line_post_reload(void)
 sccp_line_t *sccp_line_create(const char *name)
 {
 	sccp_line_t *l = NULL;
-
-	/*      // do make sure line->name is unique before adding.
-	   if ((l = sccp_line_find_byname(name, FALSE))) {
-	   sccp_line_release(l);
-	   return NULL;
-	   }
-	 */
+	
+/* 	// do make sure line->name is unique before adding.
+	if ((l = sccp_line_find_byname(name, FALSE))) {
+		sccp_line_release(l);
+		return NULL;
+	}
+*/
 	l = (sccp_line_t *) sccp_refcount_object_alloc(sizeof(sccp_line_t), SCCP_REF_LINE, name, __sccp_line_destroy);
 
 	if (!l) {
@@ -167,10 +148,10 @@ sccp_line_t *sccp_line_create(const char *name)
  */
 void sccp_line_addToGlobals(sccp_line_t * line)
 {
-	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
+	sccp_line_t *l = NULL;
 
 	SCCP_RWLIST_WRLOCK(&GLOB(lines));
-	if (l) {
+	if ((l = sccp_line_retain(line))) {
 		/* add to list */
 		l = sccp_line_retain(l);									/* add retained line to the list */
 		SCCP_RWLIST_INSERT_SORTALPHA(&GLOB(lines), l, list, cid_num);
@@ -183,6 +164,8 @@ void sccp_line_addToGlobals(sccp_line_t * line)
 		event.type = SCCP_EVENT_LINE_CREATED;
 		event.event.lineCreated.line = sccp_line_retain(l);
 		sccp_event_fire(&event);
+
+		l = sccp_line_release(l);
 	} else {
 		pbx_log(LOG_ERROR, "Adding null to global line list is not allowed!\n");
 	}
@@ -207,6 +190,7 @@ sccp_line_t *sccp_line_removeFromGlobals(sccp_line_t * line)
 	SCCP_RWLIST_WRLOCK(&GLOB(lines));
 	removed_line = SCCP_RWLIST_REMOVE(&GLOB(lines), line, list);
 	SCCP_RWLIST_UNLOCK(&GLOB(lines));
+	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Removed line '%s' from Glob(lines)\n", removed_line->name);
 
 	/* not sure if we should fire an event like this ? */
 	/*
@@ -217,7 +201,6 @@ sccp_line_t *sccp_line_removeFromGlobals(sccp_line_t * line)
 	   sccp_event_fire(&event);
 	 */
 	if (removed_line) {
-		sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Removed line '%s' from Glob(lines)\n", removed_line->name);
 		sccp_line_release(removed_line);
 	}
 
@@ -230,6 +213,7 @@ sccp_line_t *sccp_line_removeFromGlobals(sccp_line_t * line)
  */
 void *sccp_create_hotline(void)
 {
+	sccp_line_t *hotline;
 
 	GLOB(hotline) = (sccp_hotline_t *) sccp_malloc(sizeof(sccp_hotline_t));
 	if (!GLOB(hotline)) {
@@ -238,9 +222,8 @@ void *sccp_create_hotline(void)
 	}
 	memset(GLOB(hotline), 0, sizeof(sccp_hotline_t));
 
-	//SCCP_RWLIST_WRLOCK(&GLOB(lines));
-	AUTO_RELEASE sccp_line_t *hotline = sccp_line_create("Hotline");
-
+//	SCCP_RWLIST_WRLOCK(&GLOB(lines));
+	hotline = sccp_line_create("Hotline");
 	if (hotline) {
 #ifdef CS_SCCP_REALTIME
 		hotline->realtime = TRUE;
@@ -254,10 +237,12 @@ void *sccp_create_hotline(void)
 		//sccp_copy_string(hotline->mailbox, "hotline", sizeof(hotline->mailbox));
 		sccp_copy_string(GLOB(hotline)->exten, "111", sizeof(GLOB(hotline)->exten));
 
-		GLOB(hotline)->line = sccp_line_retain(hotline);						// retain line inside hotline
-		sccp_line_addToGlobals(hotline);								// retain line inside GlobalsList
+		GLOB(hotline)->line = sccp_line_retain(hotline);
+		sccp_line_addToGlobals(hotline);								// can return previous line on doubles
+		
+		sccp_line_release(hotline);
 	}
-	//SCCP_RWLIST_UNLOCK(&GLOB(lines));
+//	SCCP_RWLIST_UNLOCK(&GLOB(lines));
 
 	return NULL;
 }
@@ -279,9 +264,10 @@ void sccp_line_kill_channels(sccp_line_t * l)
 	}
 	// SCCP_LIST_LOCK(&l->channels);
 	SCCP_LIST_TRAVERSE_SAFE_BEGIN(&l->channels, c, list) {
-		AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
-
-		sccp_channel_endcall(channel);
+		if ((sccp_channel_retain(c))) {
+			sccp_channel_endcall(c);
+			sccp_channel_release(c);
+		}
 	}
 	SCCP_LIST_TRAVERSE_SAFE_END;
 	// SCCP_LIST_UNLOCK(&l->channels);
@@ -332,11 +318,9 @@ int __sccp_line_destroy(const void *ptr)
 		SCCP_LIST_HEAD_DESTROY(&l->devices);
 	}
 	// cleanup mailboxes
-	/*  
-	if (l->trnsfvm) {
-		sccp_free(l->trnsfvm);
-	}
-	*/
+//	if (l->trnsfvm) {
+//		sccp_free(l->trnsfvm);
+//	}
 	sccp_mailbox_t *mailbox = NULL;
 
 	SCCP_LIST_LOCK(&l->mailboxes);
@@ -357,18 +341,17 @@ int __sccp_line_destroy(const void *ptr)
 	if (SCCP_LIST_EMPTY(&l->mailboxes)) {
 		SCCP_LIST_HEAD_DESTROY(&l->mailboxes);
 	}
-	/*
-	#if CS_AST_HAS_NAMEDGROUP
-	if (l->namedcallgroup) {
-		sccp_free(l->namedcallgroup);
-	}
-	if (l->namedpickupgroup) {
-		sccp_free(l->namedpickupgroup);
-	}
-	#endif
-	*/
+//#if CS_AST_HAS_NAMEDGROUP
+//	if (l->namedcallgroup) {
+//		sccp_free(l->namedcallgroup);
+//	}
+//	if (l->namedpickupgroup) {
+//		sccp_free(l->namedpickupgroup);
+//	}
+//#endif
 	sccp_config_cleanup_dynamically_allocated_memory(l, SCCP_CONFIG_LINE_SEGMENT);
-
+	
+	
 	// cleanup channels
 	sccp_channel_t *c = NULL;
 
@@ -438,30 +421,6 @@ void sccp_line_delete_nolock(sccp_line_t * l)
 	sccp_line_clean(l, TRUE);
 }
 
-void sccp_line_copyCodecSetsFromLineToChannel(sccp_line_t *l, sccp_channel_t *c)
-{
-	sccp_linedevices_t *linedevice;
-	if (!l || !c) {
-		return;
-	}
-	/* combine all capabilities */
-	SCCP_LIST_LOCK(&l->devices);
-	SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
-		if (linedevice == SCCP_LIST_FIRST(&l->devices)) {
-			memcpy(&c->capabilities.audio, &linedevice->device->capabilities.audio, sizeof(c->capabilities.audio));
-			memcpy(&c->capabilities.video, &linedevice->device->capabilities.video, sizeof(c->capabilities.video));
-			memcpy(&c->preferences.audio , &linedevice->device->preferences.audio , sizeof(c->preferences.audio));
-			memcpy(&c->preferences.video , &linedevice->device->preferences.video , sizeof(c->preferences.video));
-		} else {
-			sccp_utils_combineCodecSets(&c->capabilities.audio, linedevice->device->capabilities.audio);
-			sccp_utils_combineCodecSets(&c->capabilities.video, linedevice->device->capabilities.video);
-			sccp_utils_reduceCodecSet(&c->preferences.audio , linedevice->device->preferences.audio);
-			sccp_utils_reduceCodecSet(&c->preferences.video , linedevice->device->preferences.video);
-		}
-	}
-	SCCP_LIST_UNLOCK(&l->devices);
-}
-
 /*!
  * \brief Set a Call Forward on a specific Line
  * \param line SCCP Line
@@ -478,19 +437,12 @@ void sccp_line_copyCodecSetsFromLineToChannel(sccp_line_t *l, sccp_channel_t *c)
 void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward_t type, char *number)
 {
 	sccp_linedevices_t *linedevice = NULL;
-	sccp_feature_type_t feature_type = SCCP_FEATURE_CFWDNONE;
 
 	if (!line || !device) {
 		return;
 	}
 	if ((linedevice = sccp_linedevice_find(device, line))) {
 		if (type == SCCP_CFWD_NONE) {
-			if (linedevice->cfwdAll.enabled) {
-				feature_type = SCCP_FEATURE_CFWDALL;
-			}
-			if (linedevice->cfwdBusy.enabled) {
-				feature_type = SCCP_FEATURE_CFWDBUSY;
-			}
 			linedevice->cfwdAll.enabled = 0;
 			linedevice->cfwdBusy.enabled = 0;
 			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward disabled on line %s\n", DEV_ID_LOG(device), line->name);
@@ -498,16 +450,14 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 			if (!number || sccp_strlen_zero(number)) {
 				linedevice->cfwdAll.enabled = 0;
 				linedevice->cfwdBusy.enabled = 0;
-				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward to an empty number. Invalid. Cfwd Disabled\n", DEV_ID_LOG(device));
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward to an empty number. Invalid\n", DEV_ID_LOG(device));
 			} else {
 				switch (type) {
 					case SCCP_CFWD_ALL:
-						feature_type = SCCP_FEATURE_CFWDALL;
 						linedevice->cfwdAll.enabled = 1;
 						sccp_copy_string(linedevice->cfwdAll.number, number, sizeof(linedevice->cfwdAll.number));
 						break;
 					case SCCP_CFWD_BUSY:
-						feature_type = SCCP_FEATURE_CFWDBUSY;
 						linedevice->cfwdBusy.enabled = 1;
 						sccp_copy_string(linedevice->cfwdBusy.number, number, sizeof(linedevice->cfwdBusy.number));
 						break;
@@ -515,11 +465,11 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 						linedevice->cfwdAll.enabled = 0;
 						linedevice->cfwdBusy.enabled = 0;
 				}
-				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward %s enabled on line %s to number %s\n", DEV_ID_LOG(device), sccp_callforward2str(type), line->name, number);
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward enabled on line %s to number %s\n", DEV_ID_LOG(device), line->name, number);
 			}
 		}
 		sccp_dev_starttone(linedevice->device, SKINNY_TONE_ZIPZIP, 0, 0, 0);
-		sccp_feat_changed(linedevice->device, linedevice, feature_type);
+		sccp_feat_changed(linedevice->device, linedevice, SCCP_FEATURE_CFWDALL);
 		sccp_dev_forward_status(linedevice->line, linedevice->lineInstance, device);
 		linedevice = sccp_linedevice_release(linedevice);
 	} else {
@@ -529,30 +479,38 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 
 /*!
  * \brief Attach a Device to a line
- * \param line SCCP Line
- * \param d SCCP Device
+ * \param l SCCP Line
+ * \param device SCCP Device
  * \param lineInstance lineInstance as uint8_t
  * \param subscriptionId Subscription ID for addressing individual devices on the line
  * 
  */
-void sccp_line_addDevice(sccp_line_t * line, sccp_device_t * d, uint8_t lineInstance, struct subscriptionId *subscriptionId)
+void sccp_line_addDevice(sccp_line_t * l, sccp_device_t * device, uint8_t lineInstance, struct subscriptionId *subscriptionId)
 {
-	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
-	AUTO_RELEASE sccp_device_t *device = sccp_device_retain(d);
+	sccp_linedevices_t *linedevice = NULL;
 
 	if (!device || !l) {
 		pbx_log(LOG_ERROR, "SCCP: sccp_line_addDevice: No line or device provided\n");
 		return;
 	}
 
-	sccp_linedevices_t *linedevice = NULL;
-
 	if ((linedevice = sccp_linedevice_find(device, l))) {
 		sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: device already registered for line '%s'\n", DEV_ID_LOG(device), l->name);
 		sccp_linedevice_release(linedevice);
+		// early exit
 		return;
 	}
 
+	if (!(device = sccp_device_retain(device))) {
+		pbx_log(LOG_ERROR, "SCCP: sccp_line_addDevice: Device could not be retained for line : %s\n", l ? l->name : "UNDEF");
+		return;
+	}
+
+	if (!(l = sccp_line_retain(l))) {
+		pbx_log(LOG_ERROR, "%s: sccp_line_addDevice: Line could not be retained\n", DEV_ID_LOG(device));
+		device = sccp_device_release(device);
+		return;
+	}
 	sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: add device to line %s\n", DEV_ID_LOG(device), l->name);
 
 	char ld_id[REFCOUNT_INDENTIFIER_SIZE];
@@ -577,7 +535,7 @@ void sccp_line_addDevice(sccp_line_t * line, sccp_device_t * d, uint8_t lineInst
 
 	linedevice->line->statistic.numberOfActiveDevices++;
 	linedevice->device->configurationStatistic.numberOfLines++;
-
+	
 	// fire event for new device
 	sccp_event_t event;
 
@@ -588,6 +546,9 @@ void sccp_line_addDevice(sccp_line_t * line, sccp_device_t * d, uint8_t lineInst
 
 	regcontext_exten(l, &(linedevice->subscriptionId), 1);
 	sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: added linedevice: %p with device: %s\n", l->name, linedevice, DEV_ID_LOG(device));
+
+	l = sccp_line_release(l);
+	device = sccp_device_release(device);
 }
 
 /*!
@@ -635,7 +596,7 @@ void sccp_line_removeDevice(sccp_line_t * l, sccp_device_t * device)
 /*!
  * \brief Add a Channel to a Line
  *
- * \param line SCCP Line
+ * \param l SCCP Line
  * \param channel SCCP Channel
  * 
  * \warning
@@ -647,31 +608,31 @@ void sccp_line_addChannel(sccp_line_t * line, sccp_channel_t * channel)
 	if (!line || !channel) {
 		return;
 	}
+	sccp_line_t *l = NULL;
 	sccp_channel_t *c = NULL;
 
-	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
-
-	if (l) {
+	if ((l = sccp_line_retain(line))) {
 		l->statistic.numberOfActiveChannels++;
 		SCCP_LIST_LOCK(&l->channels);
 		if ((c = sccp_channel_retain(channel))) {							// Add into list retained
 			sccp_channel_updateChannelDesignator(c);
 			sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_1 "SCCP: Adding channel %d to line %s\n", c->callid, l->name);
-			if (GLOB(callanswerorder) == SCCP_ANSWER_OLDEST_FIRST) {
-				SCCP_LIST_INSERT_TAIL(&l->channels, c, list);					// add to list
+			if (GLOB(callanswerorder) == ANSWER_OLDEST_FIRST) {
+				SCCP_LIST_INSERT_TAIL(&l->channels, c, list);
 			} else {
-				SCCP_LIST_INSERT_HEAD(&l->channels, c, list);					// add to list
-			}
+				SCCP_LIST_INSERT_HEAD(&l->channels, c, list);
+			}	
 		}
 		SCCP_LIST_UNLOCK(&l->channels);
+		l = sccp_line_release(l);
 	}
 }
 
 /*!
  * \brief Remove a Channel from a Line
  *
- * \param line SCCP Line
- * \param channel SCCP Channel
+ * \param l SCCP Line
+ * \param c SCCP Channel
  * 
  * \warning
  *  - line->channels is not always locked
@@ -683,16 +644,17 @@ void sccp_line_removeChannel(sccp_line_t * line, sccp_channel_t * channel)
 		return;
 	}
 	sccp_channel_t *c;
-	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
+	sccp_line_t *l;
 
-	if (l) {
+	if ((l = sccp_line_retain(line))) {
 		SCCP_LIST_LOCK(&l->channels);
 		if ((c = SCCP_LIST_REMOVE(&l->channels, channel, list))) {
 			sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_1 "SCCP: Removing channel %d from line %s\n", c->callid, l->name);
 			l->statistic.numberOfActiveChannels--;
-			channel = sccp_channel_release(c);							// Remove retain from list
+			channel = sccp_channel_release(c);						// Remove retain from list
 		}
 		SCCP_LIST_UNLOCK(&l->channels);
+		l = sccp_line_release(l);
 	}
 }
 
@@ -750,7 +712,7 @@ static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptio
 			} else {
 				/* un-register */
 
-				if (SCCP_LIST_GETSIZE(&l->devices) == 1) {			// only remove entry if it is the last one (shared line)
+				if (SCCP_LIST_GETSIZE(&l->devices) == 1) {							// only remove entry if it is the last one (shared line)
 					if (pbx_find_extension(NULL, NULL, &q, context, ext, 1, NULL, "", E_MATCH)) {
 						ast_context_remove_extension(context, ext, 1, NULL);
 						sccp_log((DEBUGCAT_LINE + DEBUGCAT_CONFIG)) (VERBOSE_PREFIX_1 "Unregistered RegContext: %s, Extension: %s\n", context, ext);
@@ -829,7 +791,7 @@ sccp_channelstate_t sccp_line_getDNDChannelState(sccp_line_t * line)
 sccp_line_t *sccp_line_find_byname(const char *name, uint8_t useRealtime)
 {
 	sccp_line_t *l = NULL;
-
+	
 	SCCP_RWLIST_RDLOCK(&GLOB(lines));
 	l = SCCP_RWLIST_FIND(&GLOB(lines), l, list, (sccp_strcaseequals(l->name, name)), TRUE);
 	SCCP_RWLIST_UNLOCK(&GLOB(lines));
@@ -872,7 +834,7 @@ sccp_line_t *sccp_line_find_realtime_byname(const char *name)
 #endif
 {
 	sccp_line_t *l = NULL;
-	PBX_VARIABLE_TYPE *v = NULL, *variable = NULL;
+	PBX_VARIABLE_TYPE *v, *variable;
 
 	if (sccp_strlen_zero(GLOB(realtimelinetable)) || sccp_strlen_zero(name)) {
 		return NULL;
@@ -889,16 +851,16 @@ sccp_line_t *sccp_line_find_realtime_byname(const char *name)
 
 		sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_4 "SCCP: creating realtime line '%s'\n", name);
 
-		// SCCP_RWLIST_WRLOCK(&GLOB(lines));
-		if ((l = sccp_line_create(name))) {								/* already retained */
+//		SCCP_RWLIST_WRLOCK(&GLOB(lines));
+		if ((l = sccp_line_create(name))) {									/* already retained */
 			sccp_config_applyLineConfiguration(l, variable);
 			l->realtime = TRUE;
-			sccp_line_addToGlobals(l);								// can return previous instance on doubles
+			sccp_line_addToGlobals(l);									// can return previous instance on doubles
 			pbx_variables_destroy(v);
 		} else {
 			pbx_log(LOG_ERROR, "SCCP: Unable to build realtime line '%s'\n", name);
 		}
-		// SCCP_RWLIST_UNLOCK(&GLOB(lines));
+//		SCCP_RWLIST_UNLOCK(&GLOB(lines));
 		return l;
 	}
 
@@ -936,6 +898,7 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 #endif
 {
 	sccp_line_t *l = NULL;
+	
 
 	sccp_log((DEBUGCAT_LINE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Looking for line with instance %d.\n", DEV_ID_LOG(d), instance);
 
@@ -943,11 +906,11 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 		return NULL;
 	}
 
-	if (0 < instance && instance < d->lineButtons.size && d->lineButtons.instance[instance] && d->lineButtons.instance[instance]->line) {
+	if (0 < instance && instance < d->lineButtons.size && d->lineButtons.instance[instance] && d->lineButtons.instance[instance]->line ){
 #if DEBUG
-		l = sccp_refcount_retain(d->lineButtons.instance[instance]->line, filename, lineno, func);
+		l = sccp_refcount_retain( d->lineButtons.instance[instance]->line, filename, lineno, func);
 #else
-		l = sccp_line_retain(d->lineButtons.instance[instance]->line);
+		l = sccp_line_retain( d->lineButtons.instance[instance]->line );
 #endif
 	}
 
@@ -979,8 +942,7 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const sccp_line_t * line, const char *filename, int lineno, const char *func)
 {
 	sccp_linedevices_t *linedevice = NULL;
-	sccp_line_t *l = (sccp_line_t *) line;									// loose const qualifier, to be able to lock the list;
-
+	sccp_line_t *l = (sccp_line_t *) line;       // loose const qualifier, to be able to lock the list;
 	if (!l) {
 		pbx_log(LOG_NOTICE, "%s: [%s:%d]->linedevice_find: No line provided to search in\n", DEV_ID_LOG(device), filename, lineno);
 		return NULL;
@@ -989,7 +951,7 @@ sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const s
 		pbx_log(LOG_NOTICE, "SCCP: [%s:%d]->linedevice_find: No device provided to search for (line: %s)\n", filename, lineno, line ? line->name : "UNDEF");
 		return NULL;
 	}
-
+	
 	SCCP_LIST_LOCK(&l->devices);
 	linedevice = SCCP_LIST_FIND(&l->devices, linedevice, list, (device == linedevice->device), TRUE);
 	SCCP_LIST_UNLOCK(&l->devices);
@@ -1003,7 +965,7 @@ sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const s
 sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * device, uint16_t instance, const char *filename, int lineno, const char *func)
 {
 	sccp_linedevices_t *linedevice = NULL;
-
+  
 	if (instance < 1) {
 		pbx_log(LOG_NOTICE, "%s: [%s:%d]->linedevice_find: No line provided to search in\n", DEV_ID_LOG(device), filename, lineno);
 		return NULL;
@@ -1012,9 +974,9 @@ sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * d
 		pbx_log(LOG_NOTICE, "SCCP: [%s:%d]->linedevice_find: No device provided to search for (lineinstance: %d)\n", filename, lineno, instance);
 		return NULL;
 	}
-
-	if (0 < instance && instance < device->lineButtons.size && device->lineButtons.instance[instance]) {	/* 0 < instance < lineButton.size */
-		linedevice = sccp_linedevice_retain(device->lineButtons.instance[instance]);
+	
+	if (0 < instance && instance < device->lineButtons.size && device->lineButtons.instance[instance]) {		/* 0 < instance < lineButton.size */
+		linedevice = sccp_linedevice_retain( device->lineButtons.instance[ instance ] ) ;
 	}
 
 	if (!linedevice) {
@@ -1023,44 +985,44 @@ sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * d
 	return linedevice;
 }
 
+
+
 /* create linebutton array */
-void sccp_line_createLineButtonsArray(sccp_device_t * device)
-{
+void sccp_line_createLineButtonsArray(sccp_device_t *device) {
 	sccp_linedevices_t *linedevice;
 	uint8_t lineInstances = 0;
 	btnlist *btn;
 	uint8_t i;
-
-	if (device->lineButtons.instance) {
+	
+	if(device->lineButtons.instance){
 		sccp_line_deleteLineButtonsArray(device);
 	}
-
+	
 	btn = device->buttonTemplate;
-
+	
 	for (i = 0; i < StationMaxButtonTemplateSize; i++) {
-		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].instance > lineInstances && btn[i].ptr) {
+		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].instance > lineInstances &&  btn[i].ptr) {
 			lineInstances = btn[i].instance;
-		}
+		} 
 	}
-
+	
 	device->lineButtons.size = lineInstances + SCCP_FIRST_LINEINSTANCE;					/* add the offset of SCCP_FIRST_LINEINSTANCE for explicit access */
-	device->lineButtons.instance = sccp_calloc(device->lineButtons.size, sizeof(sccp_linedevices_t *));
-
+	device->lineButtons.instance = sccp_calloc(device->lineButtons.size, sizeof(sccp_linedevices_t *) );
+	
 	for (i = 0; i < StationMaxButtonTemplateSize; i++) {
-		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].ptr) {
-			linedevice = sccp_linedevice_find(device, (sccp_line_t *) btn[i].ptr);
-			device->lineButtons.instance[btn[i].instance] = linedevice;
-		}
+		if (btn[i].type == SKINNY_BUTTONTYPE_LINE  && btn[i].ptr ) {
+			linedevice = sccp_linedevice_find(device, (sccp_line_t *)btn[i].ptr );
+			device->lineButtons.instance[ btn[i].instance ] = linedevice;
+		} 
 	}
 }
 
-void sccp_line_deleteLineButtonsArray(sccp_device_t * device)
-{
+void sccp_line_deleteLineButtonsArray(sccp_device_t *device) {
 	uint8_t i;
-
-	if (device->lineButtons.instance) {
+  
+	if(device->lineButtons.instance){
 		for (i = SCCP_FIRST_LINEINSTANCE; i < device->lineButtons.size; i++) {
-			if (device->lineButtons.instance[i]) {
+			if(device->lineButtons.instance[i]){ 
 				device->lineButtons.instance[i] = sccp_linedevice_release(device->lineButtons.instance[i]);
 			}
 		}
@@ -1068,5 +1030,3 @@ void sccp_line_deleteLineButtonsArray(sccp_device_t * device)
 		sccp_free(device->lineButtons.instance);
 	}
 }
-
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
