@@ -1,12 +1,13 @@
+
 /*!
- * \file        sccp_management.c
- * \brief       SCCP Management Class
- * \author      Marcello Ceschia <marcello [at] ceschia.de>
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \file	sccp_management.c
+ * \brief	SCCP Management Class
+ * \author	Marcello Ceschia <marcello [at] ceschia.de>
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ * 		See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $  
+ * $Date: 2010-11-22 14:09:28 +0100 (Mon, 22 Nov 2010) $
+ * $Revision: 2174 $  
  */
 #include <config.h>
 #ifdef CS_SCCP_MANAGER
@@ -21,26 +22,21 @@
 #include "sccp_actions.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
+SCCP_FILE_VERSION(__FILE__, "$Revision: 2174 $")
 
 /*
  * Descriptions
  */
-static char management_show_devices_desc[] = "Description: Lists SCCP devices in text format with details on current status. (DEPRECATED in favor of SCCPShowDevices)\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
-static char management_show_lines_desc[] = "Description: Lists SCCP lines in text format with details on current status. (DEPRECATED in favor of SCCPShowLines)\n" "\n" "LinelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
+static char management_show_devices_desc[] = "Description: Lists SCCP devices in text format with details on current status.\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
+static char management_show_lines_desc[] = "Description: Lists SCCP lines in text format with details on current status.\n" "\n" "LinelistComplete.\n" "Variables: \n" "  ActionID: <id>	Action ID for this transaction. Will be returned.\n";
 static char management_restart_devices_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device to restart\n";
 static char management_show_device_add_line_desc[] = "Description: Lists SCCP devices in text format with details on current status.\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  Devicename: Name of device to restart.\n" "  Linename: Name of line";
 static char management_device_update_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device\n";
 static char management_device_set_dnd_desc[] = "Description: set dnd on device\n" "\n" "Variables:\n" "   Devicename: Name of device\n" "  DNDState: on (busy) / off / reject/ silent";
 static char management_line_fwd_update_desc[] = "Description: update forward status for line\n" "\n" "Variables:\n" "  Devicename: Name of device\n" "  Linename: Name of line\n" "  Forwardtype: type of cfwd (all | busy | noAnswer)\n" "  Disable: yes Disable call forward (optional)\n" "  Number: number to forward calls (optional)";
-static char management_fetch_config_metadata_desc[] = "Description: fetch configuration metadata\n" "\n" "Variables:\n" "  segment: Config Segment Name (if empty returns all segments).\n";
-
-#if ASTERISK_VERSION_GROUP >= 112
-static char management_startcall_desc[] = "Description: start a new call on a device/line\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  Linename: Name of the line\n" "  number: Number to call\n" "  ChannelId: Channel UniqueId to be set on the channel\n";
-#else
+static char management_fetch_config_metadata_desc[] = "Description: fetch configuration metadata\n" "\n" "Variables:\n" "  segment: Config Segment Name (if empty returns all segments).\n" "  option: OptionName (if empty returns all options in sement).";
 static char management_startcall_desc[] = "Description: start a new call on a device/line\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  Linename: Name of the line\n" "  number: Number to call";
-#endif
-static char management_answercall_desc[] = "Description: answer a ringing channel. (DEPRECATED in favor of SCCPAnswerCall1)\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  channelId: Id of the channel to pickup\n";
+static char management_answercall_desc[] = "Description: answer a ringing channel\n" "\n" "Variables:\n" "  Devicename: Name of the Device\n" "  channelId: Id of the channel to pickup\n";
 static char management_hangupcall_desc[] = "Description: hangup a channel/call\n" "\n" "Variables:\n" "  channelId: Id of the Channel to hangup\n";
 static char management_hold_desc[] = "Description: hold/resume a call\n" "\n" "Variables:\n" "  channelId: Id of the channel to hold/unhold\n" "  hold: hold=true / resume=false\n" "  Devicename: Name of the Device\n" "  SwapChannels: Swap channels when resuming and an active channel is present (true/false)\n";
 
@@ -57,7 +53,7 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 static int sccp_manager_device_set_dnd(struct mansession *s, const struct message *m);
 static int sccp_manager_line_fwd_update(struct mansession *s, const struct message *m);
 static int sccp_manager_startCall(struct mansession *s, const struct message *m);
-static int sccp_manager_answerCall2(struct mansession *s, const struct message *m);
+static int sccp_manager_answerCall(struct mansession *s, const struct message *m);
 static int sccp_manager_hangupCall(struct mansession *s, const struct message *m);
 static int sccp_manager_holdCall(struct mansession *s, const struct message *m);
 
@@ -72,7 +68,6 @@ static struct manager_custom_hook sccp_manager_hook = {
 
 /*!
  * \brief Register management commands
- * \note deprecated
  */
 int sccp_register_management(void)
 {
@@ -85,18 +80,18 @@ int sccp_register_management(void)
 #define _MAN_FLAGS	EVENT_FLAG_SYSTEM | EVENT_FLAG_CONFIG | EVENT_FLAG_REPORTING
 #endif
 
-	result = pbx_manager_register("SCCPListDevices", _MAN_FLAGS, sccp_manager_show_devices, "List SCCP devices", management_show_devices_desc);
-	result |= pbx_manager_register("SCCPListLines", _MAN_FLAGS, sccp_manager_show_lines, "List SCCP lines", management_show_lines_desc);
+	result = pbx_manager_register("SCCPListDevices", _MAN_FLAGS, sccp_manager_show_devices, "List SCCP devices (text format)", management_show_devices_desc);
+	result |= pbx_manager_register("SCCPListLines", _MAN_FLAGS, sccp_manager_show_lines, "List SCCP lines (text format)", management_show_lines_desc);
 	result |= pbx_manager_register("SCCPDeviceRestart", _MAN_FLAGS, sccp_manager_restart_device, "Restart a given device", management_restart_devices_desc);
 	result |= pbx_manager_register("SCCPDeviceAddLine", _MAN_FLAGS, sccp_manager_device_add_line, "add a line to device", management_show_device_add_line_desc);
 	result |= pbx_manager_register("SCCPDeviceUpdate", _MAN_FLAGS, sccp_manager_device_update, "add a line to device", management_device_update_desc);
 	result |= pbx_manager_register("SCCPDeviceSetDND", _MAN_FLAGS, sccp_manager_device_set_dnd, "set dnd on device", management_device_set_dnd_desc);
 	result |= pbx_manager_register("SCCPLineForwardUpdate", _MAN_FLAGS, sccp_manager_line_fwd_update, "set call-forward on a line", management_line_fwd_update_desc);
 	result |= pbx_manager_register("SCCPStartCall", _MAN_FLAGS, sccp_manager_startCall, "start a new call on device", management_startcall_desc);
-	result |= pbx_manager_register("SCCPAnswerCall", _MAN_FLAGS, sccp_manager_answerCall2, "answer a ringin channel", management_answercall_desc);
+	result |= pbx_manager_register("SCCPAnswerCall", _MAN_FLAGS, sccp_manager_answerCall, "answer a ringin channel", management_answercall_desc);
 	result |= pbx_manager_register("SCCPHangupCall", _MAN_FLAGS, sccp_manager_hangupCall, "hangup a channel", management_hangupcall_desc);
 	result |= pbx_manager_register("SCCPHoldCall", _MAN_FLAGS, sccp_manager_holdCall, "hold/unhold a call", management_hold_desc);
-	result |= pbx_manager_register("SCCPConfigMetaData", _MAN_FLAGS, sccp_manager_config_metadata, "retrieve config metadata in json format", management_fetch_config_metadata_desc);
+	result |= pbx_manager_register("SCCPConfigMetaData", _MAN_FLAGS, sccp_manager_config_metadata, "retrieve config metadata", management_fetch_config_metadata_desc);
 #undef _MAN_FLAGS
 
 #if HAVE_PBX_MANAGER_HOOK_H
@@ -116,7 +111,6 @@ int sccp_unregister_management(void)
 	int result = 0;
 
 	result = pbx_manager_unregister("SCCPListDevices");
-	result |= pbx_manager_unregister("SCCPListLines");
 	result |= pbx_manager_unregister("SCCPDeviceRestart");
 	result |= pbx_manager_unregister("SCCPDeviceAddLine");
 	result |= pbx_manager_unregister("SCCPDeviceUpdate");
@@ -183,19 +177,21 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 		case SCCP_EVENT_DEVICE_ATTACHED:
 			device = event->event.deviceAttached.linedevice->device;				// already retained in the event
 			linedevice = event->event.deviceAttached.linedevice;					// already retained in the event
-			manager_event(EVENT_FLAG_CALL,
-				      "PeerStatus",
-				      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
-				      "ATTACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)");
+			manager_event(	EVENT_FLAG_CALL,
+					"PeerStatus",
+					"ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
+					"ATTACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)"
+					);
 			break;
 
 		case SCCP_EVENT_DEVICE_DETACHED:
 			device = event->event.deviceAttached.linedevice->device;				// already retained in the event
 			linedevice = event->event.deviceAttached.linedevice;					// already retained in the event
-			manager_event(EVENT_FLAG_CALL,
-				      "PeerStatus",
-				      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
-				      "DETACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)");
+			manager_event(	EVENT_FLAG_CALL,
+					"PeerStatus",
+					"ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
+					"DETACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)"
+					);
 			break;
 
 		case SCCP_EVENT_FEATURE_CHANGED:
@@ -205,16 +201,16 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 
 			switch (featureType) {
 				case SCCP_FEATURE_DND:
-					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", featureType2str(SCCP_FEATURE_DND), sccp_dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
+					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", featureType2str(SCCP_FEATURE_DND), dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
 					break;
 				case SCCP_FEATURE_CFWDALL:
 				case SCCP_FEATURE_CFWDBUSY:
 					if (linedevice) {
-						manager_event(EVENT_FLAG_CALL,
-							      "CallForward",
-							      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nFeature: %s\r\nStatus: %s\r\nExtension: %s\r\nSCCPLine: %s\r\nSCCPDevice: %s\r\n",
-							      featureType2str(featureType), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.enabled) ? "On" : "Off") : ((linedevice->cfwdBusy.enabled) ? "On" : "Off"), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.number) ? linedevice->cfwdAll.number : "(null)") : ((linedevice->cfwdBusy.number) ? linedevice->cfwdBusy.number : "(null)"), (linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device)
-						    );
+						manager_event(	EVENT_FLAG_CALL,
+								"CallForward",
+								"ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nFeature: %s\r\nStatus: %s\r\nExtension: %s\r\nSCCPLine: %s\r\nSCCPDevice: %s\r\n",
+								featureType2str(featureType), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.enabled) ? "On" : "Off") : ((linedevice->cfwdBusy.enabled) ? "On" : "Off"), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.number) ? linedevice->cfwdAll.number : "(null)") : ((linedevice->cfwdBusy.number) ? linedevice->cfwdBusy.number : "(null)"), (linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device)
+								);
 					}
 					break;
 				case SCCP_FEATURE_CFWDNONE:
@@ -237,7 +233,6 @@ void sccp_manager_eventListener(const sccp_event_t * event)
  * \return Success as int
  * 
  * \called_from_asterisk
- * \note deprecated
  * 
  */
 static int sccp_manager_show_devices(struct mansession *s, const struct message *m)
@@ -248,7 +243,7 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 	int total = 0;
 	struct tm *timeinfo;
 	char regtime[25];
-	char clientAddress[INET6_ADDRSTRLEN];
+        char clientAddress[INET6_ADDRSTRLEN];
 
 	snprintf(idtext, sizeof(idtext), "ActionID: %s\r\n", id);
 
@@ -257,13 +252,13 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 	SCCP_RWLIST_RDLOCK(&GLOB(devices));
 	SCCP_RWLIST_TRAVERSE(&GLOB(devices), device, list) {
 		timeinfo = localtime(&device->registrationTime);
-
-		if (device->session) {
-			sccp_copy_string(clientAddress, sccp_socket_stringify(&device->session->sin), sizeof(clientAddress));
-		} else {
-			sprintf(clientAddress, "--");
-		}
-
+                
+                if(device->session){
+                        sccp_copy_string(clientAddress, sccp_socket_stringify(&device->session->sin), sizeof(clientAddress));
+                }else {
+                        sprintf(clientAddress, "--"); 
+                }
+                
 		strftime(regtime, sizeof(regtime), "%c", timeinfo);
 		astman_append(s, "Event: DeviceEntry\r\n%s", idtext);
 		astman_append(s, "ChannelType: SCCP\r\n");
@@ -271,10 +266,10 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 		astman_append(s, "ObjectType: device\r\n");
 		astman_append(s, "Description: %s\r\n", device->description);
 		astman_append(s, "IPaddress: %s\r\n", clientAddress);
-		astman_append(s, "Reg_Status: %s\r\n", skinny_registrationstate2str(device->registrationState));
+		astman_append(s, "Reg_Status: %s\r\n", registrationstate2str(device->registrationState));
 		astman_append(s, "Reg_Time: %s\r\n", regtime);
 		astman_append(s, "Active: %s\r\n", (device->active_channel) ? "Yes" : "No");
-		astman_append(s, "NumLines: %d\r\n\r\n", device->configurationStatistic.numberOfLines);
+		astman_append(s, "NumLines: %d\r\n", device->configurationStatistic.numberOfLines);
 		total++;
 	}
 
@@ -293,7 +288,6 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
  * \return Success as int
  * 
  * \called_from_asterisk
- * \note deprecated
  * 
  */
 static int sccp_manager_show_lines(struct mansession *s, const struct message *m)
@@ -315,7 +309,7 @@ static int sccp_manager_show_lines(struct mansession *s, const struct message *m
 		astman_append(s, "ObjectType: line\r\n");
 		astman_append(s, "Name: %s\r\n", line->name);
 		astman_append(s, "Description: %s\r\n", line->description);
-		astman_append(s, "Num_Channels: %d\r\n\r\n", SCCP_RWLIST_GETSIZE(&line->channels));
+		astman_append(s, "Num_Channels: %d\r\n", SCCP_RWLIST_GETSIZE(&line->channels));
 		total++;
 	}
 
@@ -337,6 +331,7 @@ static int sccp_manager_show_lines(struct mansession *s, const struct message *m
 static int sccp_manager_restart_device(struct mansession *s, const struct message *m)
 {
 	// sccp_list_t *hintList = NULL;
+	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *type = astman_get_header(m, "Type");
 
@@ -352,8 +347,7 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 		type = "quick";
 	}
 
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
-
+	d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
@@ -361,6 +355,7 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 
 	if (!d->session) {
 		astman_send_error(s, m, "Device not registered");
+		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -373,6 +368,7 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
 	astman_send_ack(s, m, "Device restarted");
 	//astman_append(s, "Send %s restart to device %s\r\n", type, deviceName);
 	//astman_append(s, "\r\n");
+	d = sccp_device_release(d);
 
 	return 0;
 }
@@ -387,6 +383,8 @@ static int sccp_manager_restart_device(struct mansession *s, const struct messag
  */
 static int sccp_manager_device_add_line(struct mansession *s, const struct message *m)
 {
+	sccp_device_t *d = NULL;
+	sccp_line_t *line = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
 
@@ -402,22 +400,24 @@ static int sccp_manager_device_add_line(struct mansession *s, const struct messa
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
-
+	d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(lineName, TRUE);
-
+	line = sccp_line_find_byname(lineName, TRUE);
 	if (!line) {
 		astman_send_error(s, m, "Line not found");
+		d = sccp_device_release(d);
 		return 0;
 	}
 	sccp_config_addButton(d, -1, LINE, line->name, NULL, NULL);
 	astman_append(s, "Done\r\n");
 	astman_append(s, "\r\n");
+	line = sccp_line_release(line);
+	d = sccp_device_release(d);
+
 	return 0;
 }
 
@@ -431,6 +431,9 @@ static int sccp_manager_device_add_line(struct mansession *s, const struct messa
  */
 static int sccp_manager_line_fwd_update(struct mansession *s, const struct message *m)
 {
+	sccp_line_t *line = NULL;
+	sccp_device_t *d = NULL;
+	sccp_linedevices_t *linedevice = NULL;
 
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
@@ -440,31 +443,34 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 	sccp_callforward_t cfwd_type = SCCP_CFWD_NONE;
 	char cbuf[64] = "";
 
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
-
+	d = sccp_device_find_byid(deviceName, TRUE);
 	if (!d) {
 		pbx_log(LOG_WARNING, "%s: Device not found\n", deviceName);
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(lineName, TRUE);
-
+	line = sccp_line_find_byname(lineName, TRUE);
 	if (!line) {
 		pbx_log(LOG_WARNING, "%s: Line %s not found\n", deviceName, lineName);
 		astman_send_error(s, m, "Line not found");
+		d = sccp_device_release(d);
 		return 0;
 	}
 
 	if (SCCP_LIST_GETSIZE(&line->devices) > 1) {
 		pbx_log(LOG_WARNING, "%s: Callforwarding on shared lines is not supported at the moment\n", deviceName);
 		astman_send_error(s, m, "Callforwarding on shared lines is not supported at the moment");
+		line = sccp_line_release(line);
+		d = sccp_device_release(d);
 		return 0;
 	}
 
 	if (!forwardType) {
 		pbx_log(LOG_WARNING, "%s: Forwardtype is not optional [all | busy]\n", deviceName);
 		astman_send_error(s, m, "Forwardtype is not optional [all | busy]");				/* NoAnswer to be added later on */
+		line = sccp_line_release(line);
+		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -473,9 +479,7 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 	}
 
 	if (line) {
-		AUTO_RELEASE sccp_linedevices_t *linedevice = sccp_linedevice_find(d, line);
-
-		if (linedevice) {
+		if ((linedevice = sccp_linedevice_find(d, line))) {
 			if (sccp_strcaseequals("all", forwardType)) {
 				if (sccp_strcaseequals("yes", Disable)) {
 					linedevice->cfwdAll.enabled = 0;
@@ -517,13 +521,19 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
 					break;
 			}
 			sccp_dev_forward_status(line, linedevice->lineInstance, linedevice->device);
+			sccp_linedevice_release(linedevice);
 		} else {
 			pbx_log(LOG_WARNING, "%s: LineDevice not found for line %s (Device not registeed ?)\n", deviceName, lineName);
 			astman_send_error(s, m, "LineDevice not found (Device not registered ?)");
+			line = sccp_line_release(line);
+			d = sccp_device_release(d);
 			return 0;
 		}
 	}
 	astman_send_ack(s, m, cbuf);
+
+	line = sccp_line_release(line);
+	d = sccp_device_release(d);
 	return 0;
 }
 
@@ -537,6 +547,7 @@ static int sccp_manager_line_fwd_update(struct mansession *s, const struct messa
  */
 static int sccp_manager_device_update(struct mansession *s, const struct message *m)
 {
+	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 
 	if (sccp_strlen_zero(deviceName)) {
@@ -544,7 +555,7 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
+	d = sccp_device_find_byid(deviceName, FALSE);
 
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
@@ -553,6 +564,7 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 
 	if (!d->session) {
 		astman_send_error(s, m, "Device not active");
+		d = sccp_device_release(d);
 		return 0;
 	}
 
@@ -561,6 +573,8 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
 	sccp_handle_button_template_req(d->session, d, NULL);
 
 	astman_send_ack(s, m, "Done");
+
+	d = sccp_device_release(d);
 	return 0;
 }
 
@@ -574,6 +588,7 @@ static int sccp_manager_device_update(struct mansession *s, const struct message
  */
 static int sccp_manager_device_set_dnd(struct mansession *s, const struct message *m)
 {
+	sccp_device_t *d = NULL;
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *DNDState = astman_get_header(m, "DNDState");
 	int prevStatus = 0;
@@ -589,9 +604,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 		return 0;
 	}
 	//astman_append(s, "remove channel '%s' from hold\n", channelId);
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
-
-	if (d) {
+	if ((d = sccp_device_find_byid(deviceName, FALSE))) {
 		if (d->dndFeature.enabled) {
 			prevStatus = d->dndFeature.status;
 			if (sccp_strcaseequals("on", DNDState) || sccp_strcaseequals("reject", DNDState)) {
@@ -605,7 +618,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 			}
 
 			if (d->dndFeature.status != prevStatus) {
-				snprintf(retValStr, sizeof(retValStr), "Device %s DND has been set to %s", d->id, sccp_dndmode2str(d->dndFeature.status));
+				snprintf(retValStr, sizeof(retValStr), "Device %s DND has been set to %s", d->id, dndmode2str(d->dndFeature.status));
 				sccp_feat_changed(d, NULL, SCCP_FEATURE_DND);
 				sccp_dev_check_displayprompt(d);
 			} else {
@@ -614,6 +627,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 		} else {
 			astman_send_error(s, m, "DND Feature not enabled on this device.");
 		}
+		d = d ? sccp_device_release(d) : NULL;
 	} else {
 		astman_send_error(s, m, "Device could not be found.");
 		return 0;
@@ -633,19 +647,20 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
  */
 static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 {
+	sccp_device_t *d = NULL;
+	sccp_line_t *line = NULL;
+	sccp_channel_t *channel = NULL;
+
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *lineName = astman_get_header(m, "Linename");
 	const char *number = astman_get_header(m, "number");
 
-	AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
-
+	d = sccp_device_find_byid(deviceName, FALSE);
 	if (!d) {
 		astman_send_error(s, m, "Device not found");
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_line_t *line = NULL;
-
 	if (!lineName) {
 		if (d && d->defaultLineInstance > 0) {
 			line = sccp_line_find_byid(d, d->defaultLineInstance);
@@ -658,27 +673,15 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 
 	if (!line) {
 		astman_send_error(s, m, "Line not found");
+		d = sccp_device_release(d);
 		return 0;
 	}
 
-	AUTO_RELEASE sccp_channel_t *new_channel = NULL;
-
-#if ASTERISK_VERSION_GROUP >= 112
-	struct ast_assigned_ids ids = {
-		.uniqueid = astman_get_header(m, "ChannelId"),
-		//.uniqueid2 = astman_get_header(m, "OtherChannelId")
-	};
-	if ((ids.uniqueid && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid))
-	    //|| (ids.uniqueid2 && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid2))
-	    ) {
-		astman_send_error_va(s, m, "Uniqueid length exceeds maximum of %d\n", AST_MAX_PUBLIC_UNIQUEID);
-		return 0;
-	}
-	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, (ids.uniqueid) ? &ids : NULL);
-#else
-	new_channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL, NULL);
-#endif
+	channel = sccp_channel_newcall(line, d, sccp_strlen_zero(number) ? NULL : (char *) number, SKINNY_CALLTYPE_OUTBOUND, NULL);
 	astman_send_ack(s, m, "Call Started");
+	line = sccp_line_release(line);
+	d = sccp_device_release(d);
+	channel = channel ? sccp_channel_release(channel) : NULL;
 	return 0;
 }
 
@@ -690,30 +693,23 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
  * 
  * \called_from_asterisk
  */
-static int sccp_manager_answerCall2(struct mansession *s, const struct message *m)
+static int sccp_manager_answerCall(struct mansession *s, const struct message *m)
 {
+	sccp_device_t *d = NULL;
+	sccp_channel_t *c = NULL;
 	char retValStr[64] = "";
 
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *channelId = astman_get_header(m, "channelId");
 
-	if (atoi(channelId) == 0) {
+	if (atoi(channelId)==0) {
 		snprintf(retValStr, sizeof(retValStr), "Channel Id has to be a number. You have provided: '%s'\r\n", channelId);
 		astman_send_error(s, m, retValStr);
-		return 0;
+		return 0;		
 	}
-
-	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
-
-	if (c) {
-		AUTO_RELEASE sccp_device_t *d = NULL;
-
-		if (sccp_strlen_zero(deviceName)) {
-			d = sccp_channel_getDevice_retained(c);
-		} else {
-			d = sccp_device_find_byid(deviceName, FALSE);
-		}
-		if (d) {
+	
+	if ((c = sccp_channel_find_byid(atoi(channelId)))) {
+		if ((sccp_strlen_zero(deviceName) && (d = sccp_channel_getDevice_retained(c))) || (!sccp_strlen_zero(deviceName) && (d = sccp_device_find_byid(deviceName, FALSE)))) {
 			if (c->state == SCCP_CHANNELSTATE_RINGING) {
 				sccp_channel_answer(d, c);
 				if (c->owner) {
@@ -724,9 +720,11 @@ static int sccp_manager_answerCall2(struct mansession *s, const struct message *
 			} else {
 				astman_send_error(s, m, "Call is not ringing\r\n");
 			}
+			d = sccp_device_release(d);
 		} else {
 			astman_send_error(s, m, "Device not found");
 		}
+		c = sccp_channel_release(c);
 	} else {
 		astman_send_error(s, m, "Call not found\r\n");
 	}
@@ -743,15 +741,11 @@ static int sccp_manager_answerCall2(struct mansession *s, const struct message *
  */
 static int sccp_manager_hangupCall(struct mansession *s, const struct message *m)
 {
+	sccp_channel_t *c = NULL;
+
 	const char *channelId = astman_get_header(m, "channelId");
 
-	if (atoi(channelId) == 0) {
-		astman_send_error(s, m, "Channel Id has to be a number.");
-		return 0;
-	}
-
-	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
-
+	c = sccp_channel_find_byid(atoi(channelId));
 	if (!c) {
 		astman_send_error(s, m, "Call not found.");
 		return 0;
@@ -759,6 +753,7 @@ static int sccp_manager_hangupCall(struct mansession *s, const struct message *m
 	//astman_append(s, "Hangup call '%s'\r\n", channelId);
 	sccp_channel_endcall(c);
 	astman_send_ack(s, m, "Call was hungup");
+	c = sccp_channel_release(c);
 	return 0;
 }
 
@@ -772,21 +767,16 @@ static int sccp_manager_hangupCall(struct mansession *s, const struct message *m
  */
 static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 {
+	sccp_channel_t *c = NULL;
+	sccp_device_t *d = NULL;
 	const char *channelId = astman_get_header(m, "channelId");
 	const char *hold = astman_get_header(m, "hold");
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *swap = astman_get_header(m, "SwapChannels");
-	static char *retValStr = "Channel was resumed";
+	char *retValStr = "Channel was resumed";
 	boolean_t errorMessage = TRUE;
 
-	if (atoi(channelId) == 0) {
-		sprintf(retValStr, "Channel Id has to be a number. You have provided: '%s'\r\n", channelId);
-		astman_send_error(s, m, retValStr);
-		return 0;
-	}
-
-	AUTO_RELEASE sccp_channel_t *c = sccp_channel_find_byid(atoi(channelId));
-
+	c = sccp_channel_find_byid(atoi(channelId));
 	if (!c) {
 		astman_send_error(s, m, "Call not found\r\n");
 		return 0;
@@ -803,9 +793,8 @@ static int sccp_manager_holdCall(struct mansession *s, const struct message *m)
 			retValStr = "To resume a channel, you need to specify the device that resumes call using Devicename variable.";
 			goto SEND_RESPONSE;
 		}
-		AUTO_RELEASE sccp_device_t *d = sccp_device_find_byid(deviceName, FALSE);
 
-		if (d) {
+		if ((d = sccp_device_find_byid(deviceName, FALSE))) {
 			if (sccp_strcaseequals("yes", swap)) {
 				sccp_channel_resume(d, c, TRUE);
 			} else {
@@ -826,6 +815,9 @@ SEND_RESPONSE:
 	} else {
 		astman_send_ack(s, m, retValStr);
 	}
+
+	d = d ? sccp_device_release(d) : NULL;
+	c = c ? sccp_channel_release(c) : NULL;
 	return 0;
 }
 
@@ -875,6 +867,8 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 	PBX_CHANNEL_TYPE *pbxchannel = NULL;
 	PBX_CHANNEL_TYPE *pbxBridge = NULL;
 
+	sccp_channel_t *channel = NULL;
+	sccp_device_t *d = NULL;
 	char *str, *dupStr;
 
 	if (EVENT_FLAG_CALL == category) {
@@ -890,8 +884,6 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 			pbx_channel_unlock(pbxchannel);
 #endif
 
-			AUTO_RELEASE sccp_channel_t *channel = NULL;
-
 			if (pbxchannel && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxchannel))) {
 				channel = get_sccp_channel_from_pbx_channel(pbxchannel);
 			} else if (pbxchannel && ((pbxBridge = pbx_channel_get_by_name(pbx_builtin_getvar_helper(pbxchannel, "BRIDGEPEER"))) != NULL) && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxBridge))) {
@@ -902,24 +894,23 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 			}
 
 			if (channel) {
-				AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
-				if (d) {
+				if ((d = sccp_channel_getDevice_retained(channel))) {
 					if (!strcasecmp("MonitorStart", event)) {
 						d->monitorFeature.status |= SCCP_FEATURE_MONITOR_STATE_ACTIVE;
 					} else {
 						d->monitorFeature.status &= ~SCCP_FEATURE_MONITOR_STATE_ACTIVE;
 					}
 					sccp_msg_t *msg_out = NULL;
-
 					REQ(msg_out, RecordingStatusMessage);
-
+					
 					msg_out->data.RecordingStatusMessage.lel_callReference = htolel(channel->callid);
 					msg_out->data.RecordingStatusMessage.lel_status = (d->monitorFeature.status & SCCP_FEATURE_MONITOR_STATE_ACTIVE) ? htolel(1) : htolel(0);
 					sccp_dev_send(d, msg_out);
-
+					
 					sccp_feat_changed(d, NULL, SCCP_FEATURE_MONITOR);
+					d = sccp_device_release(d);
 				}
+				channel = sccp_channel_release(channel);
 			}
 		}
 	}
@@ -928,4 +919,3 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 #endif														/* HAVE_PBX_MANAGER_HOOK_H */
 
 #endif														/* CS_SCCP_MANAGER */
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
