@@ -1,12 +1,13 @@
+
 /*!
- * \file        sccp_mwi.c
- * \brief       SCCP Message Waiting Indicator Class
- * \author      Marcello Ceschia <marcello.ceschia [at] users.sourceforge.net>
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \file	sccp_mwi.c
+ * \brief	SCCP Message Waiting Indicator Class
+ * \author	Marcello Ceschia <marcello.ceschia [at] users.sourceforge.net>
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ * 		See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-05-26 17:25:57 +0200 (Tue, 26 May 2015) $
- * $Revision: 6096 $
+ * $Date: 2014-04-22 22:44:35 +0200 (Tue, 22 Apr 2014) $
+ * $Revision: 5361 $
  */
 
 #include <config.h>
@@ -17,79 +18,12 @@
 #include "sccp_line.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 6096 $");
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5361 $")
 #ifndef CS_AST_HAS_EVENT
 #define SCCP_MWI_CHECK_INTERVAL 30
 #endif
-
-
-/*!
- * \brief SCCP Mailbox Line Type Definition
- *
- * holding line information for mailbox subscription
- *
- */
-typedef struct sccp_mailboxLine sccp_mailboxLine_t;
-
-/*!
- * \brief SCCP Mailbox Line Type Structure
- *
- * holding line information for mailbox subscription
- *
- */
-struct sccp_mailboxLine {
-	sccp_line_t *line;
-	SCCP_LIST_ENTRY (sccp_mailboxLine_t) list;
-};
-
-/*!
- * \brief SCCP Mailbox Subscriber List Type Definition
- */
-typedef struct sccp_mailbox_subscriber_list sccp_mailbox_subscriber_list_t;
-
-/*!
- * \brief SCCP Mailbox Subscriber List Structure
- *
- * we hold a mailbox event subscription in sccp_mailbox_subscription_t.
- * Each line that holds a subscription for this mailbox is listed in
- */
-struct sccp_mailbox_subscriber_list {
-	char mailbox[60];
-	char context[60];
-
-	SCCP_LIST_HEAD (, sccp_mailboxLine_t) sccp_mailboxLine;
-	SCCP_LIST_ENTRY (sccp_mailbox_subscriber_list_t) list;
-
-	/*!
-	 * \brief Current Voicemail Statistic Structure
-	 */
-	struct {
-		int newmsgs;											/*!< New Messages */
-		int oldmsgs;											/*!< Old Messages */
-	} currentVoicemailStatistic;								/*!< Current Voicemail Statistic Structure */
-
-	/*!
-	 * \brief Previous Voicemail Statistic Structure
-	 */
-	struct {
-		int newmsgs;											/*!< New Messages */
-		int oldmsgs;											/*!< Old Messages */
-	} previousVoicemailStatistic;								/*!< Previous Voicemail Statistic Structure */
-
-#if defined ( CS_AST_HAS_EVENT ) || (defined( CS_AST_HAS_STASIS ))
-	/*!
-	 * \brief Asterisk Event Subscribers Structure
-	 */
-	struct pbx_event_sub *event_sub;
-#else
-	int schedUpdate;
-#endif
-};																/*!< SCCP Mailbox Subscriber List Structure */
-
-
-
-
-void sccp_mwi_setMWILineStatus(sccp_linedevices_t * lineDevice);
+void sccp_mwi_checkLine(sccp_line_t * line);
+void sccp_mwi_setMWILineStatus(sccp_linedevices_t *lineDevice);
 void sccp_mwi_linecreatedEvent(const sccp_event_t * event);
 void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event);
 void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t * line);
@@ -129,19 +63,14 @@ void sccp_mwi_module_stop(void)
 		SCCP_LIST_UNLOCK(&subscription->sccp_mailboxLine);
 		SCCP_LIST_HEAD_DESTROY(&subscription->sccp_mailboxLine);
 
+#ifdef CS_AST_HAS_EVENT
 		/* unsubscribe asterisk event */
-#if defined(CS_AST_HAS_EVENT)
 		if (subscription->event_sub) {
 			pbx_event_unsubscribe(subscription->event_sub);
 		}
-#elif defined(CS_AST_HAS_STASIS)
-		if (subscription->event_sub) {
-			stasis_unsubscribe(subscription->event_sub);
-		}
 #else
 		subscription->schedUpdate = SCCP_SCHED_DEL(subscription->schedUpdate);
 #endif
-		/* end unsubscribe asterisk event */
 
 		sccp_free(subscription);
 	}
@@ -160,12 +89,11 @@ void sccp_mwi_module_stop(void)
 static void sccp_mwi_updatecount(sccp_mailbox_subscriber_list_t * subscription)
 {
 	sccp_mailboxLine_t *mailboxLine = NULL;
+	sccp_line_t *line = NULL;
 
 	SCCP_LIST_LOCK(&subscription->sccp_mailboxLine);
 	SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {
-		AUTO_RELEASE sccp_line_t *line = sccp_line_retain(mailboxLine->line);
-
-		if (line) {
+		if ((line = sccp_line_retain(mailboxLine->line))) {
 			sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_4 "line: %s\n", line->name);
 			sccp_linedevices_t *lineDevice = NULL;
 
@@ -187,12 +115,13 @@ static void sccp_mwi_updatecount(sccp_mailbox_subscriber_list_t * subscription)
 				}
 			}
 			SCCP_LIST_UNLOCK(&line->devices);
+			sccp_line_release(line);
 		}
 	}
 	SCCP_LIST_UNLOCK(&subscription->sccp_mailboxLine);
 }
 
-#if defined(CS_AST_HAS_EVENT)
+#ifdef CS_AST_HAS_EVENT
 /*!
  * \brief Receive MWI Event from Asterisk
  * \param event Asterisk Event
@@ -205,7 +134,7 @@ void sccp_mwi_event(const struct ast_event *event, void *data)
 	pbx_log(LOG_NOTICE, "Got mwi-event\n");
 	if (!subscription || !event) {
 		return;
-	}
+        }
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "Received PBX mwi event for %s@%s\n", (subscription->mailbox) ? subscription->mailbox : "NULL", (subscription->context) ? subscription->context : "NULL");
 
 	/* for calculation store previous voicemail counts */
@@ -219,49 +148,16 @@ void sccp_mwi_event(const struct ast_event *event, void *data)
 		sccp_mwi_updatecount(subscription);
 	}
 }
-
-#elif defined(CS_AST_HAS_STASIS)
-/*!
- * \brief Receive MWI Event from Asterisk
- * \param event Asterisk Event
- * \param data Asterisk Data
- */
-
-void sccp_mwi_event(void *userdata, struct stasis_subscription *sub, struct stasis_message *msg)
-{
-	sccp_mailbox_subscriber_list_t *subscription = userdata;
-
-	sccp_log(DEBUGCAT_MWI)(VERBOSE_PREFIX_1 "Got mwi-event\n");
-	if (!subscription || !sub) {
-		return;
-	}
-
-	if (msg && ast_mwi_state_type() == stasis_message_type(msg)) {
-		struct ast_mwi_state *mwi_state = stasis_message_data(msg);
-
-		subscription->previousVoicemailStatistic.newmsgs = subscription->currentVoicemailStatistic.newmsgs;
-		subscription->previousVoicemailStatistic.oldmsgs = subscription->currentVoicemailStatistic.oldmsgs;
-
-		subscription->currentVoicemailStatistic.newmsgs = mwi_state->new_msgs;
-		subscription->currentVoicemailStatistic.oldmsgs = mwi_state->old_msgs;
-
-		if (subscription->previousVoicemailStatistic.newmsgs != subscription->currentVoicemailStatistic.newmsgs) {
-			sccp_mwi_updatecount(subscription);
-		}
-	//} else {
-	//	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "Received STASIS Message that did not contain mwi state\n");
-	}
-}
-
 #else
+
 /*!
  * \brief MWI Progress
  * \param ptr Pointer to Mailbox Subscriber list Entry
  * \note only used for asterisk version without mwi event (scheduled check)
- *
+ * 
  * \warning
  *  - line->devices is not always locked
- *
+ * 
  * \called_from_asterisk
  */
 int sccp_mwi_checksubscription(const void *ptr)
@@ -270,14 +166,14 @@ int sccp_mwi_checksubscription(const void *ptr)
 
 	if (!subscription) {
 		return -1;
-	}
+        }
 	subscription->previousVoicemailStatistic.newmsgs = subscription->currentVoicemailStatistic.newmsgs;
 	subscription->previousVoicemailStatistic.oldmsgs = subscription->currentVoicemailStatistic.oldmsgs;
 
 	char buffer[512];
 
-	snprintf(buffer, 512, "%s@%s", subscription->mailbox, subscription->context);
-	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_4 "SCCP: checking mailbox: %s\n", buffer);
+	sprintf(buffer, "%s@%s", subscription->mailbox, subscription->context);
+	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_4 "SCCP: ckecking mailbox: %s\n", buffer);
 	pbx_app_inboxcount(buffer, &subscription->currentVoicemailStatistic.newmsgs, &subscription->currentVoicemailStatistic.oldmsgs);
 
 	/* update devices if something changed */
@@ -286,7 +182,7 @@ int sccp_mwi_checksubscription(const void *ptr)
 	}
 
 	/* reschedule my self */
-	if ((subscription->schedUpdate = PBX(sched_add) (SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
+	if ((subscription->schedUpdate = sccp_sched_add(SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
 		pbx_log(LOG_ERROR, "Error creating mailbox subscription.\n");
 	}
 	return 0;
@@ -305,7 +201,7 @@ void sccp_mwi_unsubscribeMailbox(sccp_mailbox_t ** mailbox)
 }
 
 /*!
- * \brief Device Attached Event
+ * \brief Device Attached Event 
  * \param event SCCP Event
  */
 void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
@@ -316,10 +212,10 @@ void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
 	}
 
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_1 "SCCP: (mwi_deviceAttachedEvent) Get deviceAttachedEvent\n");
-
-	sccp_linedevices_t *linedevice = event->event.deviceAttached.linedevice;
-	sccp_line_t *line = linedevice->line;
-	sccp_device_t *device = linedevice->device;
+	
+	sccp_linedevices_t 	*linedevice	= event->event.deviceAttached.linedevice;
+	sccp_line_t 		*line 		= linedevice->line;
+	sccp_device_t 		*device		= linedevice->device;
 
 	if (line && device) {
 		device->voicemailStatistic.oldmsgs += line->voicemailStatistic.oldmsgs;
@@ -331,7 +227,7 @@ void sccp_mwi_deviceAttachedEvent(const sccp_event_t * event)
 }
 
 /*!
- * \brief Line Status Changed Event
+ * \brief Line Status Changed Event 
  * \param event SCCP Event
  */
 void sccp_mwi_lineStatusChangedEvent(const sccp_event_t * event)
@@ -342,20 +238,17 @@ void sccp_mwi_lineStatusChangedEvent(const sccp_event_t * event)
 	}
 
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_1 "SCCP: (mwi_lineStatusChangedEvent) Get lineStatusChangedEvent\n");
-	/* these are the only events we are interested in */
-	if (	event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_DOWN || \
-		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_ONHOOK || \
-		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_RINGING || \
-		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_OFFHOOK 
-	) {
+	if (	event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_DOWN || 
+		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_ONHOOK ||
+		event->event.lineStatusChanged.state == SCCP_CHANNELSTATE_RINGING) {				/* these are the only events we are interested in */
 		sccp_mwi_check(event->event.lineStatusChanged.optional_device);
 	}
 }
 
 /*!
- * \brief Line Created Event
+ * \brief Line Created Event 
  * \param event SCCP Event
- *
+ * 
  * \warning
  *  - line->mailboxes is not always locked
  */
@@ -366,7 +259,7 @@ void sccp_mwi_linecreatedEvent(const sccp_event_t * event)
 		return;
 	}
 
-	sccp_mailbox_t *mailbox = NULL;
+	sccp_mailbox_t *mailbox;
 	sccp_line_t *line = event->event.lineCreated.line;
 
 	sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_1 "SCCP: (mwi_linecreatedEvent) Get linecreatedEvent\n");
@@ -385,22 +278,21 @@ void sccp_mwi_linecreatedEvent(const sccp_event_t * event)
  * \param mailbox Mailbox as char
  * \param context Mailbox Context
  * \param line SCCP Line
- *
+ * 
  * \warning
  *  - subscription->sccp_mailboxLine is not always locked
  */
 void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t * line)
 {
-	if (sccp_strlen_zero(mailbox) || sccp_strlen_zero(context) || !line) {
-		pbx_log(LOG_ERROR, "%s: (addMailboxSubscription) Not all parameter contain valid pointers, mailbox: %p, context: %p\n", line ? line->name : "SCCP", mailbox, context);
-		return;
-	}
 	sccp_mailbox_subscriber_list_t *subscription = NULL;
 	sccp_mailboxLine_t *mailboxLine = NULL;
 
 	SCCP_LIST_LOCK(&sccp_mailbox_subscriptions);
 	SCCP_LIST_TRAVERSE(&sccp_mailbox_subscriptions, subscription, list) {
-		if (sccp_strequals(mailbox, subscription->mailbox) && sccp_strequals(context, subscription->context)) {
+		if (strlen(mailbox) == strlen(subscription->mailbox) &&
+		    strlen(context) == strlen(subscription->context) &&
+		    !strcmp(mailbox, subscription->mailbox) &&
+		    !strcmp(context, subscription->context)) {
 			break;
 		}
 	}
@@ -425,7 +317,6 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		SCCP_LIST_UNLOCK(&sccp_mailbox_subscriptions);
 
 		/* get initial value */
-
 #ifdef CS_AST_HAS_EVENT
 		struct ast_event *event = ast_event_get_cached(AST_EVENT_MWI,
 							       AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR, subscription->mailbox,
@@ -441,43 +332,33 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 		{												/* Fall back on checking the mailbox directly */
 			char buffer[512];
 
-			snprintf(buffer, 512, "%s@%s", subscription->mailbox, subscription->context);
+			sprintf(buffer, "%s@%s", subscription->mailbox, subscription->context);
 			pbx_app_inboxcount(buffer, &subscription->currentVoicemailStatistic.newmsgs, &subscription->currentVoicemailStatistic.oldmsgs);
 		}
 
+#ifdef CS_AST_HAS_EVENT
 		/* register asterisk event */
-#if defined( CS_AST_HAS_EVENT)
-#  if ASTERISK_VERSION_NUMBER >= 10800
+		//struct pbx_event_sub *pbx_event_subscribe(enum ast_event_type event_type, ast_event_cb_t cb, char *description, void *userdata, ...);
+#if ASTERISK_VERSION_NUMBER >= 10800
 		subscription->event_sub = pbx_event_subscribe(AST_EVENT_MWI, sccp_mwi_event, "mailbox subscription", subscription, AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR, subscription->mailbox, AST_EVENT_IE_CONTEXT, AST_EVENT_IE_PLTYPE_STR, subscription->context, AST_EVENT_IE_NEWMSGS, AST_EVENT_IE_PLTYPE_EXISTS, AST_EVENT_IE_END);
-#  else
+#else
 		subscription->event_sub = pbx_event_subscribe(AST_EVENT_MWI, sccp_mwi_event, subscription, AST_EVENT_IE_MAILBOX, AST_EVENT_IE_PLTYPE_STR, subscription->mailbox, AST_EVENT_IE_CONTEXT, AST_EVENT_IE_PLTYPE_STR, subscription->context, AST_EVENT_IE_END);
-#  endif
+#endif
 		if (!subscription->event_sub) {
 			pbx_log(LOG_ERROR, "SCCP: PBX MWI event could not be subscribed to for mailbox %s@%s\n", subscription->mailbox, subscription->context);
 		}
-#elif defined(CS_AST_HAS_STASIS)
-//		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_addMailboxSubscription) Adding STASIS Subscription for mailbox %s\n", subscription->mailbox);
-		char mailbox_context[512];
-
-		snprintf(mailbox_context, SCCP_MAX_EXTENSION + SCCP_MAX_CONTEXT + 2, "%s@%s", subscription->mailbox, subscription->context);
-		struct stasis_topic *mailbox_specific_topic = ast_mwi_topic(mailbox_context);
-		if (mailbox_specific_topic) {
-			subscription->event_sub = stasis_subscribe(mailbox_specific_topic, sccp_mwi_event, subscription);
-		}
 #else
-		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_addMailboxSubscription) Falling back to polling mailbox status\n");
-		if ((subscription->schedUpdate = PBX(sched_add) (SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
+		if ((subscription->schedUpdate = sccp_sched_add(SCCP_MWI_CHECK_INTERVAL * 1000, sccp_mwi_checksubscription, subscription)) < 0) {
 			pbx_log(LOG_ERROR, "SCCP: (mwi_addMailboxSubscription) Error creating mailbox subscription.\n");
 		}
 #endif
-		/* end register asterisk event */
 	}
 
 	/* we already have this subscription */
 	SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {
 		if (line == mailboxLine->line) {
 			break;
-		}
+                }
 	}
 
 	if (!mailboxLine) {
@@ -500,10 +381,39 @@ void sccp_mwi_addMailboxSubscription(char *mailbox, char *context, sccp_line_t *
 }
 
 /*!
- * \brief Set MWI Line Status
- * \param lineDevice SCCP LineDevice
+ * \brief Check Line for MWI Status
+ * \param line SCCP Line
  */
-void sccp_mwi_setMWILineStatus(sccp_linedevices_t * lineDevice)
+void sccp_mwi_checkLine(sccp_line_t * line)
+{
+	sccp_mailbox_t *mailbox = NULL;
+	char buffer[512];
+
+	SCCP_LIST_LOCK(&line->mailboxes);
+	SCCP_LIST_TRAVERSE(&line->mailboxes, mailbox, list) {
+		sprintf(buffer, "%s@%s", mailbox->mailbox, mailbox->context);
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_checkLine) Line: %s, Mailbox: %s\n", line->name, buffer);
+		if (!sccp_strlen_zero(buffer)) {
+
+#ifdef CS_AST_HAS_NEW_VOICEMAIL
+			pbx_app_inboxcount(buffer, &line->voicemailStatistic.newmsgs, &line->voicemailStatistic.oldmsgs);
+#else
+			if (pbx_app_has_voicemail(buffer)) {
+				line->voicemailStatistic.newmsgs = 1;
+			}
+#endif
+			sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_checkLine) Line: %s, Mailbox: %s inbox: %d/%d\n", line->name, buffer, line->voicemailStatistic.newmsgs, line->voicemailStatistic.oldmsgs);
+		}
+	}
+	SCCP_LIST_UNLOCK(&line->mailboxes);
+}
+
+/*!
+ * \brief Set MWI Line Status
+ * \param d SCCP Device
+ * \param l SCCP Line
+ */
+void sccp_mwi_setMWILineStatus(sccp_linedevices_t *lineDevice)
 {
 	sccp_msg_t *msg;
 	sccp_line_t *l = lineDevice->line;
@@ -511,47 +421,57 @@ void sccp_mwi_setMWILineStatus(sccp_linedevices_t * lineDevice)
 	int instance = 0;
 	uint8_t status = 0;
 	uint32_t mask;
+	uint32_t newState = 0;
 
-	/* when l is defined we are switching on/off the button icon, otherwise the main mwi light */
+	/* when l is defined we are switching on/off the button icon */
 	if (l) {
 		instance = lineDevice->lineInstance;
 		status = l->voicemailStatistic.newmsgs ? 1 : 0;
 	}
 
-	mask = 1 << instance;			/* mask the bit field for this line instance */
+	mask = 1 << instance;
+
+	newState = d->mwilight;
+	/* update status */
+	if (status) {
+		/* activate */
+		newState |= mask;
+	} else {
+		/* deactivate */
+		newState &= ~mask;
+	}
 
 	/* check if we need to update line status */
-	if ( (d->mwilight & mask) != (status << instance)) {
-		
-		if (status) {			/* activate mwi line icon */
-			d->mwilight |= mask;
-		} else {			/* deactivate mwi line icon */
-			d->mwilight &= ~mask;
-		}
+	if ((d->mwilight & ~(1 << 0)) != (newState & ~(1 << 0))) {
+
+		d->mwilight = newState;
 
 		REQ(msg, SetLampMessage);
 		msg->data.SetLampMessage.lel_stimulus = htolel(SKINNY_STIMULUS_VOICEMAIL);
 		msg->data.SetLampMessage.lel_stimulusInstance = htolel(instance);
-		msg->data.SetLampMessage.lel_lampMode = status ? htolel(SKINNY_LAMP_ON) : htolel(SKINNY_LAMP_OFF);
+		msg->data.SetLampMessage.lel_lampMode = (d->mwilight & ~(1 << 0)) ? htolel(d->mwilamp) : htolel(SKINNY_LAMP_OFF);
 
 		sccp_dev_send(d, msg);
-		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: (mwi_setMWILineStatus) Turn %s the MWI on line %s (%d)\n", DEV_ID_LOG(d), status ? "ON" : "OFF", (l ? l->name : "unknown"), instance);
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: (mwi_setMWILineStatus) Turn %s the MWI on line (%s)%d\n", DEV_ID_LOG(d), (mask > 0) ? "ON" : "OFF", (l ? l->name : "unknown"), instance);
 	} else {
-		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: (mwi_setMWILineStatus) Device already knows status %s on line %s (%d)\n", DEV_ID_LOG(d), status ? "ON" : "OFF", (l ? l->name : "unknown"), instance);
+		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: (mwi_setMWILineStatus) Device already knows status %s on line %s (%d)\n", DEV_ID_LOG(d), (newState & ~(1 << 0)) ? "ON" : "OFF", (l ? l->name : "unknown"), instance);
 	}
 
-	sccp_mwi_check(d); /* we need to check mwi status again, to enable/disable mwi light */
+	sccp_mwi_check(d);
 }
 
 /*!
  * \brief Check MWI Status for Device
- * \param d SCCP Device
+ * \param device SCCP Device
  * \note called by lineStatusChange
  */
-void sccp_mwi_check(sccp_device_t * d)
+void sccp_mwi_check(sccp_device_t * device)
 {
 	sccp_buttonconfig_t *config = NULL;
 
+	sccp_line_t *line = NULL;
+	sccp_channel_t *c = NULL;
+
 	sccp_msg_t *msg = NULL;
 
 	uint8_t status;
@@ -562,30 +482,26 @@ void sccp_mwi_check(sccp_device_t * d)
 	/* check if we have an active channel */
 	boolean_t hasActiveChannel = FALSE, hasRinginChannel = FALSE;
 
-	AUTO_RELEASE sccp_device_t *device = sccp_device_retain(d);
-
-	if (!device) {
+	if (!(device = sccp_device_retain(device))) {
 		sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "SCCP: (mwi_check) called with NULL device!\n");
 		return;
 	}
 
 	/* for each line, check if there is an active call */
+	sccp_device_t *tmpDevice = NULL;
+
 	SCCP_LIST_LOCK(&device->buttonconfig);
 	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
 		if (config->type == LINE) {
-			AUTO_RELEASE sccp_line_t *line = sccp_line_find_byname(config->button.line.name, FALSE);
-
+			line = sccp_line_find_byname(config->button.line.name, FALSE);
 			if (!line) {
 				sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: NULL line retrieved from buttonconfig!\n", DEV_ID_LOG(device));
 				continue;
 			}
-			sccp_channel_t *c = NULL;
-
 			SCCP_LIST_LOCK(&line->channels);
 			SCCP_LIST_TRAVERSE(&line->channels, c, list) {
-				AUTO_RELEASE sccp_device_t *tmpDevice = sccp_channel_getDevice_retained(c);
-
-				if (tmpDevice && tmpDevice == device) {						// We have a channel belonging to our device (no remote shared line channel)
+				tmpDevice = sccp_channel_getDevice_retained(c);
+				if (tmpDevice == device) {							// We have a channel belonging to our device (no remote shared line channel)
 					if (c->state != SCCP_CHANNELSTATE_ONHOOK && c->state != SCCP_CHANNELSTATE_DOWN) {
 						hasActiveChannel = TRUE;
 					}
@@ -593,12 +509,14 @@ void sccp_mwi_check(sccp_device_t * d)
 						hasRinginChannel = TRUE;
 					}
 				}
+				tmpDevice = tmpDevice ? sccp_device_release(tmpDevice) : NULL;
 			}
 			/* pre-collect number of voicemails on device to be set later */
 			oldmsgs += line->voicemailStatistic.oldmsgs;
 			newmsgs += line->voicemailStatistic.newmsgs;
 			sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: (mwi_check) line %s voicemail count %d new/%d old\n", DEV_ID_LOG(device), line->name, line->voicemailStatistic.newmsgs, line->voicemailStatistic.oldmsgs);
 			SCCP_LIST_UNLOCK(&line->channels);
+			line = sccp_line_release(line);
 		}
 	}
 	SCCP_LIST_UNLOCK(&device->buttonconfig);
@@ -619,6 +537,7 @@ void sccp_mwi_check(sccp_device_t * d)
 		} else {
 			sccp_log((DEBUGCAT_MWI)) (VERBOSE_PREFIX_3 "%s: MWI already %s on line (%s) %d\n", DEV_ID_LOG(device), "OFF", "unknown", 0);
 		}
+		device = sccp_device_release(device);
 		return;												// <---- This return must be outside the inner if
 	}
 
@@ -650,71 +569,70 @@ void sccp_mwi_check(sccp_device_t * d)
 	if (newmsgs > 0) {
 		char buffer[StationMaxDisplayTextSize];
 
-		snprintf(buffer, StationMaxDisplayTextSize, "%s: (%u/%u)", SKINNY_DISP_YOU_HAVE_VOICEMAIL, newmsgs, oldmsgs);
+		sprintf(buffer, "%s: (%d/%d)", SKINNY_DISP_YOU_HAVE_VOICEMAIL, newmsgs, oldmsgs);
 		sccp_device_addMessageToStack(device, SCCP_MESSAGE_PRIORITY_VOICEMAIL, buffer);
 	} else {
 		sccp_device_clearMessageFromStack(device, SCCP_MESSAGE_PRIORITY_VOICEMAIL);
 	}
+
+	device = sccp_device_release(device);
 }
 
 /*!
  * \brief Show MWI Subscriptions
- * \param fd Fd as int
+ * \param fd Fd as int   
  * \param total Total number of lines as int
- * \param s AMI Session
+ * \param s AMI Session 
  * \param m Message
  * \param argc Argc as int
  * \param argv[] Argv[] as char
  * \return Result as int
- *
+ * 
  * \called_from_asterisk
  */
 #include <asterisk/cli.h>
-int sccp_show_mwi_subscriptions(int fd, sccp_cli_totals_t *totals, struct mansession *s, const struct message *m, int argc, char *argv[])
+int sccp_show_mwi_subscriptions(int fd, int *total, struct mansession *s, const struct message *m, int argc, char *argv[])
 {
 	sccp_line_t *line = NULL;
 	sccp_mailboxLine_t *mailboxLine = NULL;
 	char linebuf[31] = "";
-	int local_line_total = 0;
+	int local_total = 0;
 
-#define CLI_AMI_TABLE_NAME MWISubscriptions
-#define CLI_AMI_TABLE_PER_ENTRY_NAME MailboxSubscriber
+#define CLI_AMI_TABLE_NAME MWI_Subscriptions
+#define CLI_AMI_TABLE_PER_ENTRY_NAME Mailbox_Subscriber
 #define CLI_AMI_TABLE_LIST_ITER_HEAD &sccp_mailbox_subscriptions
 #define CLI_AMI_TABLE_LIST_ITER_TYPE sccp_mailbox_subscriber_list_t
 #define CLI_AMI_TABLE_LIST_ITER_VAR subscription
 #define CLI_AMI_TABLE_LIST_LOCK SCCP_LIST_LOCK
 #define CLI_AMI_TABLE_LIST_ITERATOR SCCP_LIST_TRAVERSE
 #define CLI_AMI_TABLE_LIST_UNLOCK SCCP_LIST_UNLOCK
-#define CLI_AMI_TABLE_BEFORE_ITERATION 															\
- 		SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {								\
- 			line = mailboxLine->line;													\
- 			snprintf(linebuf,sizeof(linebuf),"%s",line->name);										\
+#define CLI_AMI_TABLE_BEFORE_ITERATION 													\
+ 		SCCP_LIST_TRAVERSE(&subscription->sccp_mailboxLine, mailboxLine, list) {						\
+ 			line = mailboxLine->line;											\
+ 			snprintf(linebuf,sizeof(linebuf),"%30s",line->name);								\
  		}
 
 #ifdef CS_AST_HAS_EVENT
-#define CLI_AMI_TABLE_FIELDS 																\
- 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10",	s,	10,	subscription->mailbox)						\
- 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30",	s,	30,	linebuf)							\
- 		CLI_AMI_TABLE_FIELD(Context,		"-15.15",	s,	15,	subscription->context)						\
- 		CLI_AMI_TABLE_FIELD(New,		"3.3",		d,	3,	subscription->currentVoicemailStatistic.newmsgs)		\
- 		CLI_AMI_TABLE_FIELD(Old,		"3.3",		d,	3,	subscription->currentVoicemailStatistic.oldmsgs)		\
- 		CLI_AMI_TABLE_FIELD(Sub,		"-3.3",		s,	3,	subscription->event_sub ? "YES" : "NO")
+#define CLI_AMI_TABLE_FIELDS 														\
+ 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10s",	10,	subscription->mailbox)				\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30s",	30,	linebuf)					\
+ 		CLI_AMI_TABLE_FIELD(Context,		"-15.15s",	15,	subscription->context)				\
+ 		CLI_AMI_TABLE_FIELD(New,		"3.3d",		3,	subscription->currentVoicemailStatistic.newmsgs)\
+ 		CLI_AMI_TABLE_FIELD(Old,		"3.3d",		3,	subscription->currentVoicemailStatistic.oldmsgs)\
+ 		CLI_AMI_TABLE_FIELD(Sub,		"-3.3s",	3,	subscription->event_sub ? "YES" : "NO")
 #include "sccp_cli_table.h"
 #else
-#define CLI_AMI_TABLE_FIELDS 																\
- 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10",	s,	10,	subscription->mailbox)						\
- 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30",	s,	30,	linebuf)							\
- 		CLI_AMI_TABLE_FIELD(Context,		"-15.15",	s,	15,	subscription->context)						\
- 		CLI_AMI_TABLE_FIELD(New,		"3.3",		d,	3,	subscription->currentVoicemailStatistic.newmsgs)		\
- 		CLI_AMI_TABLE_FIELD(Old,		"3.3",		d,	3,	subscription->currentVoicemailStatistic.oldmsgs)
+#define CLI_AMI_TABLE_FIELDS 														\
+ 		CLI_AMI_TABLE_FIELD(Mailbox,		"-10.10s",	10,	subscription->mailbox)				\
+ 		CLI_AMI_TABLE_FIELD(LineName,		"-30.30s",	30,	linebuf)					\
+ 		CLI_AMI_TABLE_FIELD(Context,		"-15.15s",	15,	subscription->context)				\
+ 		CLI_AMI_TABLE_FIELD(New,		"3.3d",		3,	subscription->currentVoicemailStatistic.newmsgs)\
+ 		CLI_AMI_TABLE_FIELD(Old,		"3.3d",		3,	subscription->currentVoicemailStatistic.oldmsgs)
 #include "sccp_cli_table.h"
 #endif
 
 	if (s) {
-		totals->lines = local_line_total;
-		totals->tables = 1;
+		*total = local_total;
 	}
 	return RESULT_SUCCESS;
 }
-
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
