@@ -1,11 +1,12 @@
+
 /*!
- * \file        sccp_refcount.c
- * \brief       SCCP Refcount Class
- * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *              See the LICENSE file at the top of the source tree.
+ * \file	sccp_refcount.c
+ * \brief	SCCP Refcount Class
+ * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ * 		See the LICENSE file at the top of the source tree.
  * 
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $  
+ * $Date$
+ * $Revision$  
  */
 
 /*!
@@ -31,7 +32,7 @@
  *
  * - Rule 4: When releasing an object the pointer we had toward the object should be nullified immediatly, either of these solutions is possible:
  *   \code
- *   d = sccp_device_release(d);                // sccp_release always returns NULL
+ *   d = sccp_device_release(d);		// sccp_release always returns NULL
  *   \endcode
  *   or 
  *   \code
@@ -39,8 +40,8 @@
  *   d = NULL;
  *   \endcode
  *   
- * - Rule 5:    You <b><em>cannnot</em></b> use free on a refcounted object. Destruction of the refcounted object and subsequent freeing of the occupied memory is performed by the sccp_release 
- *              when the number of reference reaches 0. To finalize the use of a refcounted object just release the object one final time, to negate the initial refcount of 1 during creation.
+ * - Rule 5: 	You <b><em>cannnot</em></b> use free on a refcounted object. Destruction of the refcounted object and subsequent freeing of the occupied memory is performed by the sccp_release 
+ *   		when the number of reference reaches 0. To finalize the use of a refcounted object just release the object one final time, to negate the initial refcount of 1 during creation.
  * .
  * These rules need to followed to the letter !
  */
@@ -58,9 +59,8 @@
 #include "sccp_indicate.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
-
-//nb: SCCP_HASH_PRIME defined in config.h, default 563
+SCCP_FILE_VERSION(__FILE__, "$Revision$")
+//nb: SCCP_HASH_PRIME defined in config.h, default 536
 #define SCCP_SIMPLE_HASH(_a) (((unsigned long)(_a)) % SCCP_HASH_PRIME)
 #define SCCP_LIVE_MARKER 13
 #define REF_FILE "/tmp/sccp_refs"
@@ -70,7 +70,8 @@ static struct sccp_refcount_obj_info {
 	int (*destructor) (const void *ptr);
 	char datatype[StationMaxDeviceNameSize];
 	sccp_debug_category_t debugcat;
-} obj_info[] = {
+} obj_info[] =
+{
 /* *INDENT-OFF* */
 	[SCCP_REF_DEVICE] = {NULL, "device", DEBUGCAT_DEVICE},
 	[SCCP_REF_LINE] = {NULL, "line", DEBUGCAT_LINE},
@@ -107,20 +108,10 @@ static struct refcount_objentry {
 	SCCP_RWLIST_HEAD (, RefCountedObject) refCountedObjects;						//!< one rwlock per hash table entry, used to modify list
 } *objects[SCCP_HASH_PRIME];											//!< objects hash table
 
-#if CS_REFCOUNT_DEBUG
-static FILE *sccp_ref_debug_log;
-#endif
-
 void sccp_refcount_init(void)
 {
-	sccp_log((DEBUGCAT_REFCOUNT + DEBUGCAT_HIGH)) (VERBOSE_PREFIX_1 "SCCP: (Refcount) init\n");
+	sccp_log((DEBUGCAT_REFCOUNT + DEBUGCAT_HIGH)) ("SCCP: (Refcount) init\n");
 	pbx_rwlock_init_notracking(&objectslock);								// No tracking to safe cpu cycles
-#if CS_REFCOUNT_DEBUG
-	sccp_ref_debug_log = fopen(REF_FILE, "w");
-	if (!sccp_ref_debug_log) {
-		pbx_log(LOG_NOTICE, "SCCP: Failed to open ref debug log file '%s'\n", REF_FILE);
-	}
-#endif
 	runState = SCCP_REF_RUNNING;
 }
 
@@ -154,19 +145,12 @@ void sccp_refcount_destroy(void)
 			SCCP_RWLIST_UNLOCK(&(objects[x])->refCountedObjects);
 			SCCP_RWLIST_HEAD_DESTROY(&(objects[x])->refCountedObjects);
 
-			sccp_free(objects[x]);									// free hashtable entry
+			sccp_free(objects[x]);                                                                  // free hashtable entry
 			objects[x] = NULL;
 		}
 	}
 	ast_rwlock_unlock(&objectslock);
 	pbx_rwlock_destroy(&objectslock);
-#if CS_REFCOUNT_DEBUG
-	if (sccp_ref_debug_log) {
-		fclose(sccp_ref_debug_log);
-		sccp_ref_debug_log = NULL;
-		pbx_log(LOG_NOTICE, "SCCP: ref debug log file: %s closed\n", REF_FILE);
-	}
-#endif
 	runState = SCCP_REF_DESTROYED;
 }
 
@@ -187,12 +171,7 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 	void *ptr = NULL;
 	int hash;
 
-	if (!runState) {
-		ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Not Running Yet!\n");
-		return NULL;
-	}
-
-	if (!(obj = sccp_calloc(1, size + sizeof(*obj)))) {
+	if (!(obj = calloc(1, size + sizeof(*obj)))) {
 		ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Memory allocation failure (obj)");
 		return NULL;
 	}
@@ -217,9 +196,9 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 		// create new hashtable head when necessary (should this possibly be moved to refcount_init, to avoid raceconditions ?)
 		ast_rwlock_wrlock(&objectslock);
 		if (!objects[hash]) {										// check again after getting the lock, to see if another thread did not create the head already
-			if (!(objects[hash] = sccp_malloc(sizeof(struct refcount_objentry)))) {
+			if (!(objects[hash] = malloc(sizeof(struct refcount_objentry)))) {
 				ast_log(LOG_ERROR, "SCCP: (sccp_refcount_object_alloc) Memory allocation failure (hashtable)");
-				sccp_free(obj);
+				free(obj);
 				obj = NULL;
 				ast_rwlock_unlock(&objectslock);
 				return NULL;
@@ -237,9 +216,11 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 	obj->alive = SCCP_LIVE_MARKER;
 
 #if CS_REFCOUNT_DEBUG
-	if (sccp_ref_debug_log) {
-		fprintf(sccp_ref_debug_log, "%p,+1,%d,%s,%d,%s,**constructor**,%s:%s\n", ptr, ast_get_tid(), __FILE__, __LINE__, __PRETTY_FUNCTION__, (&obj_info[obj->type])->datatype, obj->identifier);
-		fflush(sccp_ref_debug_log);
+	FILE *refo = fopen(REF_FILE, "a");
+
+	if (refo) {
+		fprintf(refo, "%p =1   %s:%d:%s (allocate new %s, len: %d, destructor: %p) [%p]\n", obj, __FILE__, __LINE__, __PRETTY_FUNCTION__, (&obj_info[obj->type])->datatype, (int) obj->len, (&obj_info[type])->destructor, ptr);
+		fclose(refo);
 	}
 #endif
 	memset(ptr, 0, size);
@@ -249,37 +230,54 @@ void *sccp_refcount_object_alloc(size_t size, enum sccp_refcounted_types type, c
 #if CS_REFCOUNT_DEBUG
 int __sccp_refcount_debug(void *ptr, RefCountedObject * obj, int delta, const char *file, int line, const char *func)
 {
-	if (!sccp_ref_debug_log) {
-		return -1;
-	}
-
 	if (ptr == NULL) {
-		fprintf(sccp_ref_debug_log, "%p **PTR IS NULL !!** %s:%d:%s\n", ptr, file, line, func);
+		FILE *refo = fopen(REF_FILE, "a");
+
+		if (refo) {
+			fprintf(refo, "%p **PTR IS NULL !!** %s:%d:%s\n", ptr, file, line, func);
+			fclose(refo);
+		}
 		// ast_assert(0);
-		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 	if (obj == NULL) {
-		fprintf(sccp_ref_debug_log, "%p **OBJ ALREADY DESTROYED !!** %s:%d:%s\n", ptr, file, line, func);
+		FILE *refo = fopen(REF_FILE, "a");
+
+		if (refo) {
+			fprintf(refo, "%p **OBJ ALREADY DESTROYED !!** %s:%d:%s\n", ptr, file, line, func);
+			fclose(refo);
+		}
 		// ast_assert(0);
-		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 
 	if (delta == 0 && obj->alive != SCCP_LIVE_MARKER) {
-		fprintf(sccp_ref_debug_log, "%p **OBJ Already destroyed and Declared DEAD !!** %s:%d:%s (%s:%s) [@%d] [%p]\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
+		FILE *refo = fopen(REF_FILE, "a");
+
+		if (refo) {
+			fprintf(refo, "%p **OBJ Already destroyed and Declared DEAD !!** %s:%d:%s (%s:%s) [@%d] [%p]\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
+			fclose(refo);
+		}
 		// ast_assert(0);
-		fflush(sccp_ref_debug_log);
 		return -1;
 	}
 
 	if (delta != 0) {
-		fprintf(sccp_ref_debug_log, "%p,%s%d,%d,%s,%d,%s,%d,%s:%s\n", ptr, (delta < 0 ? "" : "+"), delta, ast_get_tid(), file, line, func, obj ? obj->refcount : -1, (&obj_info[obj->type])->datatype, obj->identifier);
+		FILE *refo = fopen(REF_FILE, "a");
+
+		if (refo) {
+			fprintf(refo, "%p %s%d   %s:%d:%s (%s:%s) [@%d] [%p]\n", obj, (delta < 0 ? "" : "+"), delta, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier, obj->refcount, ptr);
+			fclose(refo);
+		}
 	}
 	if (obj->refcount + delta == 0 && (&obj_info[obj->type])->destructor != NULL) {
-		fprintf(sccp_ref_debug_log, "%p,%d,%d,%s,%d,%s,**destructor**,%s:%s\n", ptr, delta, ast_get_tid(), file, line, func, (&obj_info[obj->type])->datatype, obj->identifier);
+		FILE *refo = fopen(REF_FILE, "a");
+
+		if (refo) {
+			fprintf(refo, "%p **call destructor** %s:%d:%s (%s:%s)\n", ptr, file, line, func, (&obj_info[obj->type])->datatype, obj->identifier);
+			fclose(refo);
+		}
 	}
-	fflush(sccp_ref_debug_log);
 	return 0;
 }
 #endif
@@ -361,8 +359,7 @@ void sccp_refcount_print_hashtable(int fd)
 	int x, prev = 0;
 	RefCountedObject *obj = NULL;
 	unsigned int maxdepth = 0;
-	unsigned int numentries = 0;
-	float fillfactor = 0.00;
+	unsigned int fillfactor = 0;
 
 	pbx_cli(fd, "+==============================================================================================+\n");
 	pbx_cli(fd, "| %5s | %17s | %25s | %15s | %4s | %4s | %4s |\n", "hash", "type", "id", "ptr", "refc", "live", "size");
@@ -379,28 +376,25 @@ void sccp_refcount_print_hashtable(int fd)
 				}
 				pbx_cli(fd, "| %17s | %25s | %15p | %4d | %4s | %4d |\n", (obj_info[obj->type]).datatype, obj->identifier, obj, (int) obj->refcount, SCCP_LIVE_MARKER == obj->alive ? "yes" : "no", (int) obj->len);
 				prev = x;
-				numentries++;
+				fillfactor++;
 			}
 			if (maxdepth < SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects)) {
-				maxdepth = SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects);
+				maxdepth=SCCP_RWLIST_GETSIZE(&(objects[x])->refCountedObjects);
 			}
 			SCCP_RWLIST_UNLOCK(&(objects[x])->refCountedObjects);
 		}
 	}
 	ast_rwlock_unlock(&objectslock);
-	fillfactor = (float) numentries / SCCP_HASH_PRIME;
 	pbx_cli(fd, "+==============================================================================================+\n");
-	pbx_cli(fd, "| fillfactor = (%03d / %03d) = %02.2f, maxdepth = %02d                                               |\n", numentries, SCCP_HASH_PRIME, fillfactor, maxdepth);
-	if (fillfactor > 1.00) {
-		pbx_cli(fd, "| \033[1m\033[41m\033[37mPlease keep fillfactor below 1.00. Check ./configure --with-hash-size.\033[0m                       |\n");
-	}
+	pbx_cli(fd, "| fillfactor = (%03d / %03d) = %02.2f, maxdepth = %02d                                               |\n", fillfactor, SCCP_HASH_PRIME, (float)fillfactor/SCCP_HASH_PRIME, maxdepth);
 	pbx_cli(fd, "+==============================================================================================+\n");
+	
 }
 
 #ifdef CS_EXPERIMENTAL
 int sccp_refcount_force_release(long findobj, char *identifier)
 {
-	int x = 0;
+	int x= 0;
 	RefCountedObject *obj = NULL;
 	void *ptr = NULL;
 
@@ -409,7 +403,7 @@ int sccp_refcount_force_release(long findobj, char *identifier)
 		if (objects[x] && &(objects[x])->refCountedObjects) {
 			SCCP_RWLIST_RDLOCK(&(objects[x])->refCountedObjects);
 			SCCP_RWLIST_TRAVERSE(&(objects[x])->refCountedObjects, obj, list) {
-				if (sccp_strequals(obj->identifier, identifier) && (long) obj == findobj) {
+				if (sccp_strequals(obj->identifier, identifier) && (long)obj == findobj) {
 					ptr = obj->data;
 				}
 			}
@@ -498,39 +492,23 @@ gcc_inline void *sccp_refcount_release(const void *ptr, const char *filename, in
 	return NULL;
 }
 
-gcc_inline void sccp_refcount_replace(void **replaceptr, void *newptr, const char *filename, int lineno, const char *func)
-{
-	if ((!replaceptr && !newptr) || (&newptr == replaceptr)) {						// nothing changed
+gcc_inline void sccp_refcount_replace(void **replaceptr, void *newptr, const char *filename, int lineno, const char *func) {
+	if (&newptr == replaceptr) {					// nothing changed
 		return;
 	}
 
-	void *tmpNewPtr = NULL;											// retain new one first
+	void *tmpNewPtr = NULL;						// retain new one first
 	void *oldPtr = *replaceptr;
-
 	if (newptr) {
 		if ((tmpNewPtr = sccp_refcount_retain(newptr, filename, lineno, func))) {
 			*replaceptr = tmpNewPtr;
-			if (oldPtr) {										// release previous one after
+			if (oldPtr) { 							// release previous one after
 				sccp_refcount_release(oldPtr, filename, lineno, func);
 			}
 		}
 	} else {
-		if (oldPtr) {											// release previous one after
+		if (oldPtr) { 							// release previous one after
 			*replaceptr = sccp_refcount_release(oldPtr, filename, lineno, func);
 		}
 	}
 }
-
-/*
- * \brief refence autorelease helper
- * Used together with the cleanup attribute, to handle the automatic reference release of an object when we leave the scope in which the 
- * reference was defined. 
- */
-void sccp_refcount_autorelease(void *ptr)
-{
-	if (*(void **) ptr) {
-		sccp_refcount_release(*(void **) ptr, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-	}
-}
-
-// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
