@@ -1,3 +1,4 @@
+
 /*!
  * \file        sccp_rtp.c
  * \brief       SCCP RTP Class
@@ -8,8 +9,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $
+ * $Date$
+ * $Revision$
  */
 
 #include <config.h>
@@ -20,7 +21,7 @@
 #include "sccp_rtp.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
+SCCP_FILE_VERSION(__FILE__, "$Revision$")
 
 /*!
  * \brief create a new rtp server for audio data
@@ -29,11 +30,12 @@ SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
 int sccp_rtp_createAudioServer(const sccp_channel_t * c)
 {
 	boolean_t rtpResult = FALSE;
+	sccp_device_t *device = NULL;
 	boolean_t isMappedIPv4;
 
 	if (!c) {
 		return FALSE;
-	}
+        }
 	if (c->rtp.audio.rtp) {
 		sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_3 "we already have a rtp server, we use this one\n");
 		return TRUE;
@@ -50,27 +52,31 @@ int sccp_rtp_createAudioServer(const sccp_channel_t * c)
 		pbx_log(LOG_WARNING, "%s: Did not get our rtp part\n", c->currentDeviceId);
 		return FALSE;
 	}
-
+	
 	uint16_t port = sccp_rtp_getServerPort(&c->rtp.audio);
+	pbx_log(LOG_NOTICE, "RTP Server Port: %d\n", port);
 
-	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "RTP Server Port: %d\n", port);
-
-	/* depending on the client connection, we us ipv4 or ipv6 */
-	AUTO_RELEASE sccp_device_t *device = sccp_channel_getDevice_retained(c);
-
-	if (device) {
-		memcpy((void *) &c->rtp.audio.phone_remote, &device->session->ourip, sizeof(struct sockaddr_storage));
+	device = sccp_channel_getDevice_retained(c);
+	/* depeding on the client connection, we us ipv4 or ipv6*/
+	if(device){
+		memcpy((void *)&c->rtp.audio.phone_remote, &device->session->ourip, sizeof(struct sockaddr_storage));
 		sccp_socket_setPort(&c->rtp.audio.phone_remote, port);
+	}else{
+		pbx_log(LOG_NOTICE, "no device\n");
 	}
-
-	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is IPv4: %d\n", sccp_socket_is_IPv4(&c->rtp.audio.phone_remote) ? 1 : 0);
-	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is IPv6: %d\n", sccp_socket_is_IPv6(&c->rtp.audio.phone_remote) ? 1 : 0);
-
-	isMappedIPv4 = sccp_socket_ipv4_mapped(&c->rtp.audio.phone_remote, (struct sockaddr_storage *) &c->rtp.audio.phone_remote);	/*!< this is absolute necessary */
-	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is mapped: %d\n", isMappedIPv4 ? 1 : 0);
-
+	
+	
+	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is IPv4: %d\n", sccp_socket_is_IPv4(&c->rtp.audio.phone_remote) ? 1 : 0 );
+	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is IPv6: %d\n", sccp_socket_is_IPv6(&c->rtp.audio.phone_remote) ? 1 : 0 );
+	
+	
+	isMappedIPv4 = sccp_socket_ipv4_mapped(&c->rtp.audio.phone_remote, (struct sockaddr_storage *)&c->rtp.audio.phone_remote);		/*!< this is absolute necessary */
+	sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "is mapped: %d\n", isMappedIPv4 ? 1 : 0 );
+	
+	
 	//struct sockaddr_in us;
-	//PBX(rtp_setPeer) (&c->rtp.audio, &c->rtp.audio.phone, device ? device->nat : 0);
+	//      PBX(rtp_setPeer) (&c->rtp.audio, &c->rtp.audio.phone, device ? device->nat : 0);
+	device = device ? sccp_device_release(device) : NULL;
 
 	return rtpResult;
 }
@@ -85,7 +91,7 @@ int sccp_rtp_createVideoServer(const sccp_channel_t * c)
 
 	if (!c) {
 		return FALSE;
-	}
+        }
 	if (c->rtp.video.rtp) {
 		pbx_log(LOG_ERROR, "we already have a rtp server, why dont we use this?\n");
 		return TRUE;
@@ -112,7 +118,7 @@ void sccp_rtp_stop(sccp_channel_t * c)
 {
 	if (!c) {
 		return;
-	}
+        }
 	if (PBX(rtp_stop)) {
 		PBX(rtp_stop) (c);
 	} else {
@@ -139,22 +145,22 @@ void sccp_rtp_set_peer(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockaddr
 		sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_peer) remote information are equal to the current one, ignore change\n", c->currentDeviceId);
 		return;
 	}
-	//memcpy(&c->rtp.audio.phone_remote, new_peer, sizeof(c->rtp.audio.phone_remote));
-	memcpy(&rtp->phone_remote, new_peer, sizeof(rtp->phone_remote));
 
-	// inet_ntop(rtp->phone_remote.ss_family, sccp_socket_getAddr(&rtp->phone_remote), addressString, sizeof(addressString));
-	// pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s:%d\n", c->currentDeviceId, addressString, sccp_socket_getPort(new_peer));
+//	memcpy(&c->rtp.audio.phone_remote, new_peer, sizeof(c->rtp.audio.phone_remote));
+        memcpy(&rtp->phone_remote, new_peer, sizeof(rtp->phone_remote));
+        
+//        inet_ntop(rtp->phone_remote.ss_family, sccp_socket_getAddr(&rtp->phone_remote), addressString, sizeof(addressString));
+//	pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s:%d\n", c->currentDeviceId, addressString, sccp_socket_getPort(new_peer));
 	pbx_log(LOG_NOTICE, "%s: ( sccp_rtp_set_peer ) Set remote address to %s\n", c->currentDeviceId, sccp_socket_stringify(&rtp->phone_remote));
 
 	if (rtp->readState & SCCP_RTP_STATUS_ACTIVE) {
 		/* Shutdown any early-media or previous media on re-invite */
 		/*! \todo we should wait for the acknowledgement to get back. We don't have a function/procedure in place to do this at this moment in time (sccp_dev_send_wait) */
 		sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_peer) Stop media transmission on channel %d\n", c->currentDeviceId, c->callid);
-
-		/*! \todo we should check if this is a video or autio rtp */
-		// sccp_channel_stopmediatransmission(c);
-		// sccp_channel_startMediaTransmission(c);
-		sccp_channel_updateMediaTransmission(c);
+                
+                /*! \todo we should check if this is a video or autio rtp */
+//		sccp_channel_stopmediatransmission(c);
+		sccp_channel_startMediaTransmission(c);
 	}
 
 }
@@ -167,47 +173,39 @@ void sccp_rtp_set_peer(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockaddr
  */
 void sccp_rtp_set_phone(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockaddr_storage *new_peer)
 {
+	sccp_device_t *device;
+
 	/* validate socket */
-	if (sccp_socket_getPort(new_peer) == 0) {
+	if ( sccp_socket_getPort(new_peer) == 0) {
 		sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_phone) remote information are invalid, dont change anything\n", c->currentDeviceId);
 		return;
 	}
 
-	AUTO_RELEASE sccp_device_t *device = sccp_channel_getDevice_retained(c);
-
-	if (device) {
+	if ((device = sccp_channel_getDevice_retained(c))) {
 
 		/* check if we have new infos */
 		/*! \todo if we enable this, we get an audio issue when resume on the same device, so we need to force asterisk to update -MC */
 		/*
-		if (socket_equals(new_peer, &c->rtp.audio.phone)) {
-			sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_phone) remote information are equal to the current one, ignore change\n", c->currentDeviceId);
-			//return;
-		} 
-		*/
+		   if (socket_equals(new_peer, &c->rtp.audio.phone)) {
+		   sccp_log((DEBUGCAT_RTP)) (VERBOSE_PREFIX_2 "%s: (sccp_rtp_set_phone) remote information are equal to the current one, ignore change\n", c->currentDeviceId);
+		   //return;
+		   } 
+		 */
 
 		memcpy(&rtp->phone, new_peer, sizeof(rtp->phone));
 
 		//update pbx
 		if (PBX(rtp_setPhoneAddress)) {
-#ifndef CS_EXPERIMENTAL
 			PBX(rtp_setPhoneAddress) (rtp, new_peer, device->nat);
-#else
-			PBX(rtp_setPhoneAddress) (rtp, new_peer, device->nat >= SCCP_NAT_ON ? 1 : 0);
-#endif
 		}
 
-		char buf1[NI_MAXHOST + NI_MAXSERV];
-
-		sccp_copy_string(buf1, sccp_socket_stringify(&rtp->phone_remote), sizeof(buf1));
-		char buf2[NI_MAXHOST + NI_MAXSERV];
-
-		sccp_copy_string(buf2, sccp_socket_stringify(&rtp->phone), sizeof(buf2));
-#ifndef CS_EXPERIMENTAL
+                char buf1[NI_MAXHOST + NI_MAXSERV];
+                sccp_copy_string(buf1, sccp_socket_stringify(&rtp->phone_remote), sizeof(buf1));
+                char buf2[NI_MAXHOST + NI_MAXSERV];
+                sccp_copy_string(buf2, sccp_socket_stringify(&rtp->phone), sizeof(buf2));
 		sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "%s: Tell PBX   to send RTP/UDP media from %s to %s (NAT: %s)\n", DEV_ID_LOG(device), buf1, buf2, device->nat ? "yes" : "no");
-#else
-		sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "%s: Tell PBX   to send RTP/UDP media from %s to %s (NAT: %s)\n", DEV_ID_LOG(device), buf1, buf2, sccp_nat2str(device->nat));
-#endif
+
+		device = sccp_device_release(device);
 	}
 }
 
@@ -216,10 +214,10 @@ void sccp_rtp_set_phone(sccp_channel_t * c, struct sccp_rtp *rtp, struct sockadd
  */
 sccp_rtp_info_t sccp_rtp_getAudioPeerInfo(const sccp_channel_t * c, struct sccp_rtp **rtp)
 {
+	sccp_device_t *device = NULL;
 	sccp_rtp_info_t result = SCCP_RTP_INFO_NORTP;
 
-	AUTO_RELEASE sccp_device_t *device = sccp_channel_getDevice_retained(c);
-
+	device = sccp_channel_getDevice_retained(c);
 	if (!device) {
 		return SCCP_RTP_INFO_NORTP;
 	}
@@ -228,13 +226,11 @@ sccp_rtp_info_t sccp_rtp_getAudioPeerInfo(const sccp_channel_t * c, struct sccp_
 
 	result = SCCP_RTP_INFO_AVAILABLE;
 	// \todo add apply_ha(d->ha, &sin) check here instead
-#ifndef CS_EXPERIMENTAL
 	if (device->directrtp && !device->nat && !c->conference) {
-#else
-	if (device->directrtp && device->nat == SCCP_NAT_OFF && !c->conference) {
-#endif
 		result |= SCCP_RTP_INFO_ALLOW_DIRECTRTP;
 	}
+
+	device = device ? sccp_device_release(device) : NULL;
 	return result;
 }
 
@@ -244,9 +240,9 @@ sccp_rtp_info_t sccp_rtp_getAudioPeerInfo(const sccp_channel_t * c, struct sccp_
 sccp_rtp_info_t sccp_rtp_getVideoPeerInfo(const sccp_channel_t * c, struct sccp_rtp ** rtp)
 {
 	sccp_rtp_info_t result = SCCP_RTP_INFO_NORTP;
+	sccp_device_t *device = NULL;
 
-	AUTO_RELEASE sccp_device_t *device = sccp_channel_getDevice_retained(c);
-
+	device = sccp_channel_getDevice_retained(c);
 	if (!device) {
 		return SCCP_RTP_INFO_NORTP;
 	}
@@ -254,13 +250,11 @@ sccp_rtp_info_t sccp_rtp_getVideoPeerInfo(const sccp_channel_t * c, struct sccp_
 	*rtp = &(((sccp_channel_t *) c)->rtp.video);
 
 	result = SCCP_RTP_INFO_AVAILABLE;
-#ifndef CS_EXPERIMENTAL
 	if (device->directrtp && !device->nat && !c->conference) {
-#else
-	if (device->directrtp && device->nat == SCCP_NAT_OFF && !c->conference) {
-#endif
 		result |= SCCP_RTP_INFO_ALLOW_DIRECTRTP;
 	}
+
+	device = device ? sccp_device_release(device) : NULL;
 	return result;
 }
 
@@ -294,7 +288,9 @@ int sccp_rtp_get_sampleRate(skinny_codec_t codec)
  */
 void sccp_rtp_destroy(sccp_channel_t * c)
 {
-	sccp_line_t *l = c->line;
+	sccp_line_t *l = NULL;
+
+	l = c->line;
 
 	if (c->rtp.audio.rtp) {
 		sccp_log(DEBUGCAT_RTP) (VERBOSE_PREFIX_3 "%s: destroying PBX rtp server on channel %s-%08X\n", c->currentDeviceId, l ? l->name : "(null)", c->callid);
@@ -336,20 +332,18 @@ boolean_t sccp_rtp_getUs(const struct sccp_rtp *rtp, struct sockaddr_storage *us
 		PBX(rtp_getUs) (rtp->rtp, us);
 		return TRUE;
 	} else {
-		// us = &rtp->phone_remote;
+//		us = &rtp->phone_remote;
 		return FALSE;
 	}
 }
 
-uint16_t sccp_rtp_getServerPort(const struct sccp_rtp * rtp)
-{
-	uint16_t port = 0;
-	struct sockaddr_storage sas;
-
-	sccp_rtp_getUs(rtp, &sas);
-
-	port = sccp_socket_getPort(&sas);
-	return port;
+uint16_t sccp_rtp_getServerPort(const struct sccp_rtp *rtp){
+    uint16_t port = 0;
+    struct sockaddr_storage sas;
+    sccp_rtp_getUs(rtp, &sas);
+    
+    port = sccp_socket_getPort(&sas);
+    return port;
 }
 
 /*!
